/*
 * Copyright 2023 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.ai.generativelanguage.v1;

import com.google.ai.generativelanguage.v1.stub.GenerativeServiceStub;
import com.google.ai.generativelanguage.v1.stub.GenerativeServiceStubSettings;
import com.google.api.gax.core.BackgroundResource;
import com.google.api.gax.rpc.ServerStreamingCallable;
import com.google.api.gax.rpc.UnaryCallable;
import java.io.IOException;
import java.util.List;
import java.util.concurrent.TimeUnit;
import javax.annotation.Generated;

// AUTO-GENERATED DOCUMENTATION AND CLASS.
/**
 * Service Description: API for using Large Models that generate multimodal content and have
 * additional capabilities beyond text generation.
 *
 * <p>This class provides the ability to make remote calls to the backing service through method
 * calls that map to API methods. Sample code to get started:
 *
 * <pre>{@code
 * // This snippet has been automatically generated and should be regarded as a code template only.
 * // It will require modifications to work:
 * // - It may require correct/in-range values for request initialization.
 * // - It may require specifying regional endpoints when creating the service client as shown in
 * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
 * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
 *   ModelName model = ModelName.of("[MODEL]");
 *   List<Content> contents = new ArrayList<>();
 *   GenerateContentResponse response = generativeServiceClient.generateContent(model, contents);
 * }
 * }</pre>
 *
 * <p>Note: close() needs to be called on the GenerativeServiceClient object to clean up resources
 * such as threads. In the example above, try-with-resources is used, which automatically calls
 * close().
 *
 * <p>The surface of this class includes several types of Java methods for each of the API's
 * methods:
 *
 * <ol>
 *   <li>A "flattened" method. With this type of method, the fields of the request type have been
 *       converted into function parameters. It may be the case that not all fields are available as
 *       parameters, and not every API method will have a flattened method entry point.
 *   <li>A "request object" method. This type of method only takes one parameter, a request object,
 *       which must be constructed before the call. Not every API method will have a request object
 *       method.
 *   <li>A "callable" method. This type of method takes no parameters and returns an immutable API
 *       callable object, which can be used to initiate calls to the service.
 * </ol>
 *
 * <p>See the individual methods for example code.
 *
 * <p>Many parameters require resource names to be formatted in a particular way. To assist with
 * these names, this class includes a format method for each type of name, and additionally a parse
 * method to extract the individual identifiers contained within names that are returned.
 *
 * <p>This class can be customized by passing in a custom instance of GenerativeServiceSettings to
 * create(). For example:
 *
 * <p>To customize credentials:
 *
 * <pre>{@code
 * // This snippet has been automatically generated and should be regarded as a code template only.
 * // It will require modifications to work:
 * // - It may require correct/in-range values for request initialization.
 * // - It may require specifying regional endpoints when creating the service client as shown in
 * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
 * GenerativeServiceSettings generativeServiceSettings =
 *     GenerativeServiceSettings.newBuilder()
 *         .setCredentialsProvider(FixedCredentialsProvider.create(myCredentials))
 *         .build();
 * GenerativeServiceClient generativeServiceClient =
 *     GenerativeServiceClient.create(generativeServiceSettings);
 * }</pre>
 *
 * <p>To customize the endpoint:
 *
 * <pre>{@code
 * // This snippet has been automatically generated and should be regarded as a code template only.
 * // It will require modifications to work:
 * // - It may require correct/in-range values for request initialization.
 * // - It may require specifying regional endpoints when creating the service client as shown in
 * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
 * GenerativeServiceSettings generativeServiceSettings =
 *     GenerativeServiceSettings.newBuilder().setEndpoint(myEndpoint).build();
 * GenerativeServiceClient generativeServiceClient =
 *     GenerativeServiceClient.create(generativeServiceSettings);
 * }</pre>
 *
 * <p>To use REST (HTTP1.1/JSON) transport (instead of gRPC) for sending and receiving requests over
 * the wire:
 *
 * <pre>{@code
 * // This snippet has been automatically generated and should be regarded as a code template only.
 * // It will require modifications to work:
 * // - It may require correct/in-range values for request initialization.
 * // - It may require specifying regional endpoints when creating the service client as shown in
 * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
 * GenerativeServiceSettings generativeServiceSettings =
 *     GenerativeServiceSettings.newHttpJsonBuilder().build();
 * GenerativeServiceClient generativeServiceClient =
 *     GenerativeServiceClient.create(generativeServiceSettings);
 * }</pre>
 *
 * <p>Please refer to the GitHub repository's samples for more quickstart code snippets.
 */
@Generated("by gapic-generator-java")
public class GenerativeServiceClient implements BackgroundResource {
  private final GenerativeServiceSettings settings;
  private final GenerativeServiceStub stub;

  /** Constructs an instance of GenerativeServiceClient with default settings. */
  public static final GenerativeServiceClient create() throws IOException {
    return create(GenerativeServiceSettings.newBuilder().build());
  }

  /**
   * Constructs an instance of GenerativeServiceClient, using the given settings. The channels are
   * created based on the settings passed in, or defaults for any settings that are not set.
   */
  public static final GenerativeServiceClient create(GenerativeServiceSettings settings)
      throws IOException {
    return new GenerativeServiceClient(settings);
  }

  /**
   * Constructs an instance of GenerativeServiceClient, using the given stub for making calls. This
   * is for advanced usage - prefer using create(GenerativeServiceSettings).
   */
  public static final GenerativeServiceClient create(GenerativeServiceStub stub) {
    return new GenerativeServiceClient(stub);
  }

  /**
   * Constructs an instance of GenerativeServiceClient, using the given settings. This is protected
   * so that it is easy to make a subclass, but otherwise, the static factory methods should be
   * preferred.
   */
  protected GenerativeServiceClient(GenerativeServiceSettings settings) throws IOException {
    this.settings = settings;
    this.stub = ((GenerativeServiceStubSettings) settings.getStubSettings()).createStub();
  }

  protected GenerativeServiceClient(GenerativeServiceStub stub) {
    this.settings = null;
    this.stub = stub;
  }

  public final GenerativeServiceSettings getSettings() {
    return settings;
  }

  public GenerativeServiceStub getStub() {
    return stub;
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates a response from the model given an input `GenerateContentRequest`.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   ModelName model = ModelName.of("[MODEL]");
   *   List<Content> contents = new ArrayList<>();
   *   GenerateContentResponse response = generativeServiceClient.generateContent(model, contents);
   * }
   * }</pre>
   *
   * @param model Required. The name of the `Model` to use for generating the completion.
   *     <p>Format: `name=models/{model}`.
   * @param contents Required. The content of the current conversation with the model.
   *     <p>For single-turn queries, this is a single instance. For multi-turn queries, this is a
   *     repeated field that contains conversation history + latest request.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final GenerateContentResponse generateContent(ModelName model, List<Content> contents) {
    GenerateContentRequest request =
        GenerateContentRequest.newBuilder()
            .setModel(model == null ? null : model.toString())
            .addAllContents(contents)
            .build();
    return generateContent(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates a response from the model given an input `GenerateContentRequest`.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   String model = ModelName.of("[MODEL]").toString();
   *   List<Content> contents = new ArrayList<>();
   *   GenerateContentResponse response = generativeServiceClient.generateContent(model, contents);
   * }
   * }</pre>
   *
   * @param model Required. The name of the `Model` to use for generating the completion.
   *     <p>Format: `name=models/{model}`.
   * @param contents Required. The content of the current conversation with the model.
   *     <p>For single-turn queries, this is a single instance. For multi-turn queries, this is a
   *     repeated field that contains conversation history + latest request.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final GenerateContentResponse generateContent(String model, List<Content> contents) {
    GenerateContentRequest request =
        GenerateContentRequest.newBuilder().setModel(model).addAllContents(contents).build();
    return generateContent(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates a response from the model given an input `GenerateContentRequest`.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   GenerateContentRequest request =
   *       GenerateContentRequest.newBuilder()
   *           .setModel(ModelName.of("[MODEL]").toString())
   *           .addAllContents(new ArrayList<Content>())
   *           .addAllSafetySettings(new ArrayList<SafetySetting>())
   *           .setGenerationConfig(GenerationConfig.newBuilder().build())
   *           .build();
   *   GenerateContentResponse response = generativeServiceClient.generateContent(request);
   * }
   * }</pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final GenerateContentResponse generateContent(GenerateContentRequest request) {
    return generateContentCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates a response from the model given an input `GenerateContentRequest`.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   GenerateContentRequest request =
   *       GenerateContentRequest.newBuilder()
   *           .setModel(ModelName.of("[MODEL]").toString())
   *           .addAllContents(new ArrayList<Content>())
   *           .addAllSafetySettings(new ArrayList<SafetySetting>())
   *           .setGenerationConfig(GenerationConfig.newBuilder().build())
   *           .build();
   *   ApiFuture<GenerateContentResponse> future =
   *       generativeServiceClient.generateContentCallable().futureCall(request);
   *   // Do something.
   *   GenerateContentResponse response = future.get();
   * }
   * }</pre>
   */
  public final UnaryCallable<GenerateContentRequest, GenerateContentResponse>
      generateContentCallable() {
    return stub.generateContentCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates a streamed response from the model given an input `GenerateContentRequest`.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   GenerateContentRequest request =
   *       GenerateContentRequest.newBuilder()
   *           .setModel(ModelName.of("[MODEL]").toString())
   *           .addAllContents(new ArrayList<Content>())
   *           .addAllSafetySettings(new ArrayList<SafetySetting>())
   *           .setGenerationConfig(GenerationConfig.newBuilder().build())
   *           .build();
   *   ServerStream<GenerateContentResponse> stream =
   *       generativeServiceClient.streamGenerateContentCallable().call(request);
   *   for (GenerateContentResponse response : stream) {
   *     // Do something when a response is received.
   *   }
   * }
   * }</pre>
   */
  public final ServerStreamingCallable<GenerateContentRequest, GenerateContentResponse>
      streamGenerateContentCallable() {
    return stub.streamGenerateContentCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates an embedding from the model given an input `Content`.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   ModelName model = ModelName.of("[MODEL]");
   *   Content content = Content.newBuilder().build();
   *   EmbedContentResponse response = generativeServiceClient.embedContent(model, content);
   * }
   * }</pre>
   *
   * @param model Required. The model's resource name. This serves as an ID for the Model to use.
   *     <p>This name should match a model name returned by the `ListModels` method.
   *     <p>Format: `models/{model}`
   * @param content Required. The content to embed. Only the `parts.text` fields will be counted.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final EmbedContentResponse embedContent(ModelName model, Content content) {
    EmbedContentRequest request =
        EmbedContentRequest.newBuilder()
            .setModel(model == null ? null : model.toString())
            .setContent(content)
            .build();
    return embedContent(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates an embedding from the model given an input `Content`.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   String model = ModelName.of("[MODEL]").toString();
   *   Content content = Content.newBuilder().build();
   *   EmbedContentResponse response = generativeServiceClient.embedContent(model, content);
   * }
   * }</pre>
   *
   * @param model Required. The model's resource name. This serves as an ID for the Model to use.
   *     <p>This name should match a model name returned by the `ListModels` method.
   *     <p>Format: `models/{model}`
   * @param content Required. The content to embed. Only the `parts.text` fields will be counted.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final EmbedContentResponse embedContent(String model, Content content) {
    EmbedContentRequest request =
        EmbedContentRequest.newBuilder().setModel(model).setContent(content).build();
    return embedContent(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates an embedding from the model given an input `Content`.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   EmbedContentRequest request =
   *       EmbedContentRequest.newBuilder()
   *           .setModel(ModelName.of("[MODEL]").toString())
   *           .setContent(Content.newBuilder().build())
   *           .setTaskType(TaskType.forNumber(0))
   *           .setTitle("title110371416")
   *           .build();
   *   EmbedContentResponse response = generativeServiceClient.embedContent(request);
   * }
   * }</pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final EmbedContentResponse embedContent(EmbedContentRequest request) {
    return embedContentCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates an embedding from the model given an input `Content`.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   EmbedContentRequest request =
   *       EmbedContentRequest.newBuilder()
   *           .setModel(ModelName.of("[MODEL]").toString())
   *           .setContent(Content.newBuilder().build())
   *           .setTaskType(TaskType.forNumber(0))
   *           .setTitle("title110371416")
   *           .build();
   *   ApiFuture<EmbedContentResponse> future =
   *       generativeServiceClient.embedContentCallable().futureCall(request);
   *   // Do something.
   *   EmbedContentResponse response = future.get();
   * }
   * }</pre>
   */
  public final UnaryCallable<EmbedContentRequest, EmbedContentResponse> embedContentCallable() {
    return stub.embedContentCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates multiple embeddings from the model given input text in a synchronous call.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   ModelName model = ModelName.of("[MODEL]");
   *   List<EmbedContentRequest> requests = new ArrayList<>();
   *   BatchEmbedContentsResponse response =
   *       generativeServiceClient.batchEmbedContents(model, requests);
   * }
   * }</pre>
   *
   * @param model Required. The model's resource name. This serves as an ID for the Model to use.
   *     <p>This name should match a model name returned by the `ListModels` method.
   *     <p>Format: `models/{model}`
   * @param requests Required. Embed requests for the batch. The model in each of these requests
   *     must match the model specified `BatchEmbedContentsRequest.model`.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final BatchEmbedContentsResponse batchEmbedContents(
      ModelName model, List<EmbedContentRequest> requests) {
    BatchEmbedContentsRequest request =
        BatchEmbedContentsRequest.newBuilder()
            .setModel(model == null ? null : model.toString())
            .addAllRequests(requests)
            .build();
    return batchEmbedContents(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates multiple embeddings from the model given input text in a synchronous call.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   String model = ModelName.of("[MODEL]").toString();
   *   List<EmbedContentRequest> requests = new ArrayList<>();
   *   BatchEmbedContentsResponse response =
   *       generativeServiceClient.batchEmbedContents(model, requests);
   * }
   * }</pre>
   *
   * @param model Required. The model's resource name. This serves as an ID for the Model to use.
   *     <p>This name should match a model name returned by the `ListModels` method.
   *     <p>Format: `models/{model}`
   * @param requests Required. Embed requests for the batch. The model in each of these requests
   *     must match the model specified `BatchEmbedContentsRequest.model`.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final BatchEmbedContentsResponse batchEmbedContents(
      String model, List<EmbedContentRequest> requests) {
    BatchEmbedContentsRequest request =
        BatchEmbedContentsRequest.newBuilder().setModel(model).addAllRequests(requests).build();
    return batchEmbedContents(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates multiple embeddings from the model given input text in a synchronous call.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   BatchEmbedContentsRequest request =
   *       BatchEmbedContentsRequest.newBuilder()
   *           .setModel(ModelName.of("[MODEL]").toString())
   *           .addAllRequests(new ArrayList<EmbedContentRequest>())
   *           .build();
   *   BatchEmbedContentsResponse response = generativeServiceClient.batchEmbedContents(request);
   * }
   * }</pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final BatchEmbedContentsResponse batchEmbedContents(BatchEmbedContentsRequest request) {
    return batchEmbedContentsCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Generates multiple embeddings from the model given input text in a synchronous call.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   BatchEmbedContentsRequest request =
   *       BatchEmbedContentsRequest.newBuilder()
   *           .setModel(ModelName.of("[MODEL]").toString())
   *           .addAllRequests(new ArrayList<EmbedContentRequest>())
   *           .build();
   *   ApiFuture<BatchEmbedContentsResponse> future =
   *       generativeServiceClient.batchEmbedContentsCallable().futureCall(request);
   *   // Do something.
   *   BatchEmbedContentsResponse response = future.get();
   * }
   * }</pre>
   */
  public final UnaryCallable<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
      batchEmbedContentsCallable() {
    return stub.batchEmbedContentsCallable();
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Runs a model's tokenizer on input content and returns the token count.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   ModelName model = ModelName.of("[MODEL]");
   *   List<Content> contents = new ArrayList<>();
   *   CountTokensResponse response = generativeServiceClient.countTokens(model, contents);
   * }
   * }</pre>
   *
   * @param model Required. The model's resource name. This serves as an ID for the Model to use.
   *     <p>This name should match a model name returned by the `ListModels` method.
   *     <p>Format: `models/{model}`
   * @param contents Required. The input given to the model as a prompt.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final CountTokensResponse countTokens(ModelName model, List<Content> contents) {
    CountTokensRequest request =
        CountTokensRequest.newBuilder()
            .setModel(model == null ? null : model.toString())
            .addAllContents(contents)
            .build();
    return countTokens(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Runs a model's tokenizer on input content and returns the token count.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   String model = ModelName.of("[MODEL]").toString();
   *   List<Content> contents = new ArrayList<>();
   *   CountTokensResponse response = generativeServiceClient.countTokens(model, contents);
   * }
   * }</pre>
   *
   * @param model Required. The model's resource name. This serves as an ID for the Model to use.
   *     <p>This name should match a model name returned by the `ListModels` method.
   *     <p>Format: `models/{model}`
   * @param contents Required. The input given to the model as a prompt.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final CountTokensResponse countTokens(String model, List<Content> contents) {
    CountTokensRequest request =
        CountTokensRequest.newBuilder().setModel(model).addAllContents(contents).build();
    return countTokens(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Runs a model's tokenizer on input content and returns the token count.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   CountTokensRequest request =
   *       CountTokensRequest.newBuilder()
   *           .setModel(ModelName.of("[MODEL]").toString())
   *           .addAllContents(new ArrayList<Content>())
   *           .build();
   *   CountTokensResponse response = generativeServiceClient.countTokens(request);
   * }
   * }</pre>
   *
   * @param request The request object containing all of the parameters for the API call.
   * @throws com.google.api.gax.rpc.ApiException if the remote call fails
   */
  public final CountTokensResponse countTokens(CountTokensRequest request) {
    return countTokensCallable().call(request);
  }

  // AUTO-GENERATED DOCUMENTATION AND METHOD.
  /**
   * Runs a model's tokenizer on input content and returns the token count.
   *
   * <p>Sample code:
   *
   * <pre>{@code
   * // This snippet has been automatically generated and should be regarded as a code template only.
   * // It will require modifications to work:
   * // - It may require correct/in-range values for request initialization.
   * // - It may require specifying regional endpoints when creating the service client as shown in
   * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
   * try (GenerativeServiceClient generativeServiceClient = GenerativeServiceClient.create()) {
   *   CountTokensRequest request =
   *       CountTokensRequest.newBuilder()
   *           .setModel(ModelName.of("[MODEL]").toString())
   *           .addAllContents(new ArrayList<Content>())
   *           .build();
   *   ApiFuture<CountTokensResponse> future =
   *       generativeServiceClient.countTokensCallable().futureCall(request);
   *   // Do something.
   *   CountTokensResponse response = future.get();
   * }
   * }</pre>
   */
  public final UnaryCallable<CountTokensRequest, CountTokensResponse> countTokensCallable() {
    return stub.countTokensCallable();
  }

  @Override
  public final void close() {
    stub.close();
  }

  @Override
  public void shutdown() {
    stub.shutdown();
  }

  @Override
  public boolean isShutdown() {
    return stub.isShutdown();
  }

  @Override
  public boolean isTerminated() {
    return stub.isTerminated();
  }

  @Override
  public void shutdownNow() {
    stub.shutdownNow();
  }

  @Override
  public boolean awaitTermination(long duration, TimeUnit unit) throws InterruptedException {
    return stub.awaitTermination(duration, unit);
  }
}
