/*
 * Copyright 2023 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.ai.generativelanguage.v1;

import com.google.ai.generativelanguage.v1.stub.GenerativeServiceStubSettings;
import com.google.api.core.ApiFunction;
import com.google.api.core.BetaApi;
import com.google.api.gax.core.GoogleCredentialsProvider;
import com.google.api.gax.core.InstantiatingExecutorProvider;
import com.google.api.gax.grpc.InstantiatingGrpcChannelProvider;
import com.google.api.gax.httpjson.InstantiatingHttpJsonChannelProvider;
import com.google.api.gax.rpc.ApiClientHeaderProvider;
import com.google.api.gax.rpc.ClientContext;
import com.google.api.gax.rpc.ClientSettings;
import com.google.api.gax.rpc.ServerStreamingCallSettings;
import com.google.api.gax.rpc.StubSettings;
import com.google.api.gax.rpc.TransportChannelProvider;
import com.google.api.gax.rpc.UnaryCallSettings;
import java.io.IOException;
import java.util.List;
import javax.annotation.Generated;

// AUTO-GENERATED DOCUMENTATION AND CLASS.
/**
 * Settings class to configure an instance of {@link GenerativeServiceClient}.
 *
 * <p>The default instance has everything set to sensible defaults:
 *
 * <ul>
 *   <li>The default service address (generativelanguage.googleapis.com) and default port (443) are
 *       used.
 *   <li>Credentials are acquired automatically through Application Default Credentials.
 *   <li>Retries are configured for idempotent methods but not for non-idempotent methods.
 * </ul>
 *
 * <p>The builder of this class is recursive, so contained classes are themselves builders. When
 * build() is called, the tree of builders is called to create the complete settings object.
 *
 * <p>For example, to set the total timeout of generateContent to 30 seconds:
 *
 * <pre>{@code
 * // This snippet has been automatically generated and should be regarded as a code template only.
 * // It will require modifications to work:
 * // - It may require correct/in-range values for request initialization.
 * // - It may require specifying regional endpoints when creating the service client as shown in
 * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
 * GenerativeServiceSettings.Builder generativeServiceSettingsBuilder =
 *     GenerativeServiceSettings.newBuilder();
 * generativeServiceSettingsBuilder
 *     .generateContentSettings()
 *     .setRetrySettings(
 *         generativeServiceSettingsBuilder
 *             .generateContentSettings()
 *             .getRetrySettings()
 *             .toBuilder()
 *             .setTotalTimeout(Duration.ofSeconds(30))
 *             .build());
 * GenerativeServiceSettings generativeServiceSettings = generativeServiceSettingsBuilder.build();
 * }</pre>
 */
@Generated("by gapic-generator-java")
public class GenerativeServiceSettings extends ClientSettings<GenerativeServiceSettings> {

  /** Returns the object with the settings used for calls to generateContent. */
  public UnaryCallSettings<GenerateContentRequest, GenerateContentResponse>
      generateContentSettings() {
    return ((GenerativeServiceStubSettings) getStubSettings()).generateContentSettings();
  }

  /** Returns the object with the settings used for calls to streamGenerateContent. */
  public ServerStreamingCallSettings<GenerateContentRequest, GenerateContentResponse>
      streamGenerateContentSettings() {
    return ((GenerativeServiceStubSettings) getStubSettings()).streamGenerateContentSettings();
  }

  /** Returns the object with the settings used for calls to embedContent. */
  public UnaryCallSettings<EmbedContentRequest, EmbedContentResponse> embedContentSettings() {
    return ((GenerativeServiceStubSettings) getStubSettings()).embedContentSettings();
  }

  /** Returns the object with the settings used for calls to batchEmbedContents. */
  public UnaryCallSettings<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
      batchEmbedContentsSettings() {
    return ((GenerativeServiceStubSettings) getStubSettings()).batchEmbedContentsSettings();
  }

  /** Returns the object with the settings used for calls to countTokens. */
  public UnaryCallSettings<CountTokensRequest, CountTokensResponse> countTokensSettings() {
    return ((GenerativeServiceStubSettings) getStubSettings()).countTokensSettings();
  }

  public static final GenerativeServiceSettings create(GenerativeServiceStubSettings stub)
      throws IOException {
    return new GenerativeServiceSettings.Builder(stub.toBuilder()).build();
  }

  /** Returns a builder for the default ExecutorProvider for this service. */
  public static InstantiatingExecutorProvider.Builder defaultExecutorProviderBuilder() {
    return GenerativeServiceStubSettings.defaultExecutorProviderBuilder();
  }

  /** Returns the default service endpoint. */
  public static String getDefaultEndpoint() {
    return GenerativeServiceStubSettings.getDefaultEndpoint();
  }

  /** Returns the default service scopes. */
  public static List<String> getDefaultServiceScopes() {
    return GenerativeServiceStubSettings.getDefaultServiceScopes();
  }

  /** Returns a builder for the default credentials for this service. */
  public static GoogleCredentialsProvider.Builder defaultCredentialsProviderBuilder() {
    return GenerativeServiceStubSettings.defaultCredentialsProviderBuilder();
  }

  /** Returns a builder for the default gRPC ChannelProvider for this service. */
  public static InstantiatingGrpcChannelProvider.Builder defaultGrpcTransportProviderBuilder() {
    return GenerativeServiceStubSettings.defaultGrpcTransportProviderBuilder();
  }

  /** Returns a builder for the default REST ChannelProvider for this service. */
  @BetaApi
  public static InstantiatingHttpJsonChannelProvider.Builder
      defaultHttpJsonTransportProviderBuilder() {
    return GenerativeServiceStubSettings.defaultHttpJsonTransportProviderBuilder();
  }

  public static TransportChannelProvider defaultTransportChannelProvider() {
    return GenerativeServiceStubSettings.defaultTransportChannelProvider();
  }

  @BetaApi("The surface for customizing headers is not stable yet and may change in the future.")
  public static ApiClientHeaderProvider.Builder defaultApiClientHeaderProviderBuilder() {
    return GenerativeServiceStubSettings.defaultApiClientHeaderProviderBuilder();
  }

  /** Returns a new gRPC builder for this class. */
  public static Builder newBuilder() {
    return Builder.createDefault();
  }

  /** Returns a new REST builder for this class. */
  @BetaApi
  public static Builder newHttpJsonBuilder() {
    return Builder.createHttpJsonDefault();
  }

  /** Returns a new builder for this class. */
  public static Builder newBuilder(ClientContext clientContext) {
    return new Builder(clientContext);
  }

  /** Returns a builder containing all the values of this settings class. */
  public Builder toBuilder() {
    return new Builder(this);
  }

  protected GenerativeServiceSettings(Builder settingsBuilder) throws IOException {
    super(settingsBuilder);
  }

  /** Builder for GenerativeServiceSettings. */
  public static class Builder extends ClientSettings.Builder<GenerativeServiceSettings, Builder> {

    protected Builder() throws IOException {
      this(((ClientContext) null));
    }

    protected Builder(ClientContext clientContext) {
      super(GenerativeServiceStubSettings.newBuilder(clientContext));
    }

    protected Builder(GenerativeServiceSettings settings) {
      super(settings.getStubSettings().toBuilder());
    }

    protected Builder(GenerativeServiceStubSettings.Builder stubSettings) {
      super(stubSettings);
    }

    private static Builder createDefault() {
      return new Builder(GenerativeServiceStubSettings.newBuilder());
    }

    @BetaApi
    private static Builder createHttpJsonDefault() {
      return new Builder(GenerativeServiceStubSettings.newHttpJsonBuilder());
    }

    public GenerativeServiceStubSettings.Builder getStubSettingsBuilder() {
      return ((GenerativeServiceStubSettings.Builder) getStubSettings());
    }

    /**
     * Applies the given settings updater function to all of the unary API methods in this service.
     *
     * <p>Note: This method does not support applying settings to streaming methods.
     */
    public Builder applyToAllUnaryMethods(
        ApiFunction<UnaryCallSettings.Builder<?, ?>, Void> settingsUpdater) {
      super.applyToAllUnaryMethods(
          getStubSettingsBuilder().unaryMethodSettingsBuilders(), settingsUpdater);
      return this;
    }

    /** Returns the builder for the settings used for calls to generateContent. */
    public UnaryCallSettings.Builder<GenerateContentRequest, GenerateContentResponse>
        generateContentSettings() {
      return getStubSettingsBuilder().generateContentSettings();
    }

    /** Returns the builder for the settings used for calls to streamGenerateContent. */
    public ServerStreamingCallSettings.Builder<GenerateContentRequest, GenerateContentResponse>
        streamGenerateContentSettings() {
      return getStubSettingsBuilder().streamGenerateContentSettings();
    }

    /** Returns the builder for the settings used for calls to embedContent. */
    public UnaryCallSettings.Builder<EmbedContentRequest, EmbedContentResponse>
        embedContentSettings() {
      return getStubSettingsBuilder().embedContentSettings();
    }

    /** Returns the builder for the settings used for calls to batchEmbedContents. */
    public UnaryCallSettings.Builder<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
        batchEmbedContentsSettings() {
      return getStubSettingsBuilder().batchEmbedContentsSettings();
    }

    /** Returns the builder for the settings used for calls to countTokens. */
    public UnaryCallSettings.Builder<CountTokensRequest, CountTokensResponse>
        countTokensSettings() {
      return getStubSettingsBuilder().countTokensSettings();
    }

    @Override
    public GenerativeServiceSettings build() throws IOException {
      return new GenerativeServiceSettings(this);
    }
  }
}
