/*
 * Copyright 2023 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.ai.generativelanguage.v1.stub;

import com.google.ai.generativelanguage.v1.BatchEmbedContentsRequest;
import com.google.ai.generativelanguage.v1.BatchEmbedContentsResponse;
import com.google.ai.generativelanguage.v1.CountTokensRequest;
import com.google.ai.generativelanguage.v1.CountTokensResponse;
import com.google.ai.generativelanguage.v1.EmbedContentRequest;
import com.google.ai.generativelanguage.v1.EmbedContentResponse;
import com.google.ai.generativelanguage.v1.GenerateContentRequest;
import com.google.ai.generativelanguage.v1.GenerateContentResponse;
import com.google.api.core.ApiFunction;
import com.google.api.core.BetaApi;
import com.google.api.gax.core.GaxProperties;
import com.google.api.gax.core.GoogleCredentialsProvider;
import com.google.api.gax.core.InstantiatingExecutorProvider;
import com.google.api.gax.grpc.GaxGrpcProperties;
import com.google.api.gax.grpc.GrpcTransportChannel;
import com.google.api.gax.grpc.InstantiatingGrpcChannelProvider;
import com.google.api.gax.httpjson.GaxHttpJsonProperties;
import com.google.api.gax.httpjson.HttpJsonTransportChannel;
import com.google.api.gax.httpjson.InstantiatingHttpJsonChannelProvider;
import com.google.api.gax.retrying.RetrySettings;
import com.google.api.gax.rpc.ApiClientHeaderProvider;
import com.google.api.gax.rpc.ClientContext;
import com.google.api.gax.rpc.ServerStreamingCallSettings;
import com.google.api.gax.rpc.StatusCode;
import com.google.api.gax.rpc.StubSettings;
import com.google.api.gax.rpc.TransportChannelProvider;
import com.google.api.gax.rpc.UnaryCallSettings;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Lists;
import java.io.IOException;
import java.util.List;
import javax.annotation.Generated;
import org.threeten.bp.Duration;

// AUTO-GENERATED DOCUMENTATION AND CLASS.
/**
 * Settings class to configure an instance of {@link GenerativeServiceStub}.
 *
 * <p>The default instance has everything set to sensible defaults:
 *
 * <ul>
 *   <li>The default service address (generativelanguage.googleapis.com) and default port (443) are
 *       used.
 *   <li>Credentials are acquired automatically through Application Default Credentials.
 *   <li>Retries are configured for idempotent methods but not for non-idempotent methods.
 * </ul>
 *
 * <p>The builder of this class is recursive, so contained classes are themselves builders. When
 * build() is called, the tree of builders is called to create the complete settings object.
 *
 * <p>For example, to set the total timeout of generateContent to 30 seconds:
 *
 * <pre>{@code
 * // This snippet has been automatically generated and should be regarded as a code template only.
 * // It will require modifications to work:
 * // - It may require correct/in-range values for request initialization.
 * // - It may require specifying regional endpoints when creating the service client as shown in
 * // https://cloud.google.com/java/docs/setup#configure_endpoints_for_the_client_library
 * GenerativeServiceStubSettings.Builder generativeServiceSettingsBuilder =
 *     GenerativeServiceStubSettings.newBuilder();
 * generativeServiceSettingsBuilder
 *     .generateContentSettings()
 *     .setRetrySettings(
 *         generativeServiceSettingsBuilder
 *             .generateContentSettings()
 *             .getRetrySettings()
 *             .toBuilder()
 *             .setTotalTimeout(Duration.ofSeconds(30))
 *             .build());
 * GenerativeServiceStubSettings generativeServiceSettings =
 *     generativeServiceSettingsBuilder.build();
 * }</pre>
 */
@Generated("by gapic-generator-java")
public class GenerativeServiceStubSettings extends StubSettings<GenerativeServiceStubSettings> {
  /** The default scopes of the service. */
  private static final ImmutableList<String> DEFAULT_SERVICE_SCOPES =
      ImmutableList.<String>builder().build();

  private final UnaryCallSettings<GenerateContentRequest, GenerateContentResponse>
      generateContentSettings;
  private final ServerStreamingCallSettings<GenerateContentRequest, GenerateContentResponse>
      streamGenerateContentSettings;
  private final UnaryCallSettings<EmbedContentRequest, EmbedContentResponse> embedContentSettings;
  private final UnaryCallSettings<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
      batchEmbedContentsSettings;
  private final UnaryCallSettings<CountTokensRequest, CountTokensResponse> countTokensSettings;

  /** Returns the object with the settings used for calls to generateContent. */
  public UnaryCallSettings<GenerateContentRequest, GenerateContentResponse>
      generateContentSettings() {
    return generateContentSettings;
  }

  /** Returns the object with the settings used for calls to streamGenerateContent. */
  public ServerStreamingCallSettings<GenerateContentRequest, GenerateContentResponse>
      streamGenerateContentSettings() {
    return streamGenerateContentSettings;
  }

  /** Returns the object with the settings used for calls to embedContent. */
  public UnaryCallSettings<EmbedContentRequest, EmbedContentResponse> embedContentSettings() {
    return embedContentSettings;
  }

  /** Returns the object with the settings used for calls to batchEmbedContents. */
  public UnaryCallSettings<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
      batchEmbedContentsSettings() {
    return batchEmbedContentsSettings;
  }

  /** Returns the object with the settings used for calls to countTokens. */
  public UnaryCallSettings<CountTokensRequest, CountTokensResponse> countTokensSettings() {
    return countTokensSettings;
  }

  public GenerativeServiceStub createStub() throws IOException {
    if (getTransportChannelProvider()
        .getTransportName()
        .equals(GrpcTransportChannel.getGrpcTransportName())) {
      return GrpcGenerativeServiceStub.create(this);
    }
    if (getTransportChannelProvider()
        .getTransportName()
        .equals(HttpJsonTransportChannel.getHttpJsonTransportName())) {
      return HttpJsonGenerativeServiceStub.create(this);
    }
    throw new UnsupportedOperationException(
        String.format(
            "Transport not supported: %s", getTransportChannelProvider().getTransportName()));
  }

  /** Returns a builder for the default ExecutorProvider for this service. */
  public static InstantiatingExecutorProvider.Builder defaultExecutorProviderBuilder() {
    return InstantiatingExecutorProvider.newBuilder();
  }

  /** Returns the default service endpoint. */
  public static String getDefaultEndpoint() {
    return "generativelanguage.googleapis.com:443";
  }

  /** Returns the default mTLS service endpoint. */
  public static String getDefaultMtlsEndpoint() {
    return "generativelanguage.mtls.googleapis.com:443";
  }

  /** Returns the default service scopes. */
  public static List<String> getDefaultServiceScopes() {
    return DEFAULT_SERVICE_SCOPES;
  }

  /** Returns a builder for the default credentials for this service. */
  public static GoogleCredentialsProvider.Builder defaultCredentialsProviderBuilder() {
    return GoogleCredentialsProvider.newBuilder()
        .setScopesToApply(DEFAULT_SERVICE_SCOPES)
        .setUseJwtAccessWithScope(true);
  }

  /** Returns a builder for the default gRPC ChannelProvider for this service. */
  public static InstantiatingGrpcChannelProvider.Builder defaultGrpcTransportProviderBuilder() {
    return InstantiatingGrpcChannelProvider.newBuilder()
        .setMaxInboundMessageSize(Integer.MAX_VALUE);
  }

  /** Returns a builder for the default REST ChannelProvider for this service. */
  @BetaApi
  public static InstantiatingHttpJsonChannelProvider.Builder
      defaultHttpJsonTransportProviderBuilder() {
    return InstantiatingHttpJsonChannelProvider.newBuilder();
  }

  public static TransportChannelProvider defaultTransportChannelProvider() {
    return defaultGrpcTransportProviderBuilder().build();
  }

  @BetaApi("The surface for customizing headers is not stable yet and may change in the future.")
  public static ApiClientHeaderProvider.Builder defaultGrpcApiClientHeaderProviderBuilder() {
    return ApiClientHeaderProvider.newBuilder()
        .setGeneratedLibToken(
            "gapic", GaxProperties.getLibraryVersion(GenerativeServiceStubSettings.class))
        .setTransportToken(
            GaxGrpcProperties.getGrpcTokenName(), GaxGrpcProperties.getGrpcVersion());
  }

  @BetaApi("The surface for customizing headers is not stable yet and may change in the future.")
  public static ApiClientHeaderProvider.Builder defaultHttpJsonApiClientHeaderProviderBuilder() {
    return ApiClientHeaderProvider.newBuilder()
        .setGeneratedLibToken(
            "gapic", GaxProperties.getLibraryVersion(GenerativeServiceStubSettings.class))
        .setTransportToken(
            GaxHttpJsonProperties.getHttpJsonTokenName(),
            GaxHttpJsonProperties.getHttpJsonVersion());
  }

  public static ApiClientHeaderProvider.Builder defaultApiClientHeaderProviderBuilder() {
    return GenerativeServiceStubSettings.defaultGrpcApiClientHeaderProviderBuilder();
  }

  /** Returns a new gRPC builder for this class. */
  public static Builder newBuilder() {
    return Builder.createDefault();
  }

  /** Returns a new REST builder for this class. */
  public static Builder newHttpJsonBuilder() {
    return Builder.createHttpJsonDefault();
  }

  /** Returns a new builder for this class. */
  public static Builder newBuilder(ClientContext clientContext) {
    return new Builder(clientContext);
  }

  /** Returns a builder containing all the values of this settings class. */
  public Builder toBuilder() {
    return new Builder(this);
  }

  protected GenerativeServiceStubSettings(Builder settingsBuilder) throws IOException {
    super(settingsBuilder);

    generateContentSettings = settingsBuilder.generateContentSettings().build();
    streamGenerateContentSettings = settingsBuilder.streamGenerateContentSettings().build();
    embedContentSettings = settingsBuilder.embedContentSettings().build();
    batchEmbedContentsSettings = settingsBuilder.batchEmbedContentsSettings().build();
    countTokensSettings = settingsBuilder.countTokensSettings().build();
  }

  /** Builder for GenerativeServiceStubSettings. */
  public static class Builder extends StubSettings.Builder<GenerativeServiceStubSettings, Builder> {
    private final ImmutableList<UnaryCallSettings.Builder<?, ?>> unaryMethodSettingsBuilders;
    private final UnaryCallSettings.Builder<GenerateContentRequest, GenerateContentResponse>
        generateContentSettings;
    private final ServerStreamingCallSettings.Builder<
            GenerateContentRequest, GenerateContentResponse>
        streamGenerateContentSettings;
    private final UnaryCallSettings.Builder<EmbedContentRequest, EmbedContentResponse>
        embedContentSettings;
    private final UnaryCallSettings.Builder<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
        batchEmbedContentsSettings;
    private final UnaryCallSettings.Builder<CountTokensRequest, CountTokensResponse>
        countTokensSettings;
    private static final ImmutableMap<String, ImmutableSet<StatusCode.Code>>
        RETRYABLE_CODE_DEFINITIONS;

    static {
      ImmutableMap.Builder<String, ImmutableSet<StatusCode.Code>> definitions =
          ImmutableMap.builder();
      definitions.put(
          "retry_policy_0_codes",
          ImmutableSet.copyOf(Lists.<StatusCode.Code>newArrayList(StatusCode.Code.UNAVAILABLE)));
      RETRYABLE_CODE_DEFINITIONS = definitions.build();
    }

    private static final ImmutableMap<String, RetrySettings> RETRY_PARAM_DEFINITIONS;

    static {
      ImmutableMap.Builder<String, RetrySettings> definitions = ImmutableMap.builder();
      RetrySettings settings = null;
      settings =
          RetrySettings.newBuilder()
              .setInitialRetryDelay(Duration.ofMillis(1000L))
              .setRetryDelayMultiplier(1.3)
              .setMaxRetryDelay(Duration.ofMillis(10000L))
              .setInitialRpcTimeout(Duration.ofMillis(60000L))
              .setRpcTimeoutMultiplier(1.0)
              .setMaxRpcTimeout(Duration.ofMillis(60000L))
              .setTotalTimeout(Duration.ofMillis(60000L))
              .build();
      definitions.put("retry_policy_0_params", settings);
      RETRY_PARAM_DEFINITIONS = definitions.build();
    }

    protected Builder() {
      this(((ClientContext) null));
    }

    protected Builder(ClientContext clientContext) {
      super(clientContext);

      generateContentSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();
      streamGenerateContentSettings = ServerStreamingCallSettings.newBuilder();
      embedContentSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();
      batchEmbedContentsSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();
      countTokensSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      unaryMethodSettingsBuilders =
          ImmutableList.<UnaryCallSettings.Builder<?, ?>>of(
              generateContentSettings,
              embedContentSettings,
              batchEmbedContentsSettings,
              countTokensSettings);
      initDefaults(this);
    }

    protected Builder(GenerativeServiceStubSettings settings) {
      super(settings);

      generateContentSettings = settings.generateContentSettings.toBuilder();
      streamGenerateContentSettings = settings.streamGenerateContentSettings.toBuilder();
      embedContentSettings = settings.embedContentSettings.toBuilder();
      batchEmbedContentsSettings = settings.batchEmbedContentsSettings.toBuilder();
      countTokensSettings = settings.countTokensSettings.toBuilder();

      unaryMethodSettingsBuilders =
          ImmutableList.<UnaryCallSettings.Builder<?, ?>>of(
              generateContentSettings,
              embedContentSettings,
              batchEmbedContentsSettings,
              countTokensSettings);
    }

    private static Builder createDefault() {
      Builder builder = new Builder(((ClientContext) null));

      builder.setTransportChannelProvider(defaultTransportChannelProvider());
      builder.setCredentialsProvider(defaultCredentialsProviderBuilder().build());
      builder.setInternalHeaderProvider(defaultApiClientHeaderProviderBuilder().build());
      builder.setEndpoint(getDefaultEndpoint());
      builder.setMtlsEndpoint(getDefaultMtlsEndpoint());
      builder.setSwitchToMtlsEndpointAllowed(true);

      return initDefaults(builder);
    }

    private static Builder createHttpJsonDefault() {
      Builder builder = new Builder(((ClientContext) null));

      builder.setTransportChannelProvider(defaultHttpJsonTransportProviderBuilder().build());
      builder.setCredentialsProvider(defaultCredentialsProviderBuilder().build());
      builder.setInternalHeaderProvider(defaultHttpJsonApiClientHeaderProviderBuilder().build());
      builder.setEndpoint(getDefaultEndpoint());
      builder.setMtlsEndpoint(getDefaultMtlsEndpoint());
      builder.setSwitchToMtlsEndpointAllowed(true);

      return initDefaults(builder);
    }

    private static Builder initDefaults(Builder builder) {
      builder
          .generateContentSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("retry_policy_0_codes"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("retry_policy_0_params"));

      builder
          .streamGenerateContentSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("retry_policy_0_codes"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("retry_policy_0_params"));

      builder
          .embedContentSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("retry_policy_0_codes"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("retry_policy_0_params"));

      builder
          .batchEmbedContentsSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("retry_policy_0_codes"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("retry_policy_0_params"));

      builder
          .countTokensSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("retry_policy_0_codes"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("retry_policy_0_params"));

      return builder;
    }

    /**
     * Applies the given settings updater function to all of the unary API methods in this service.
     *
     * <p>Note: This method does not support applying settings to streaming methods.
     */
    public Builder applyToAllUnaryMethods(
        ApiFunction<UnaryCallSettings.Builder<?, ?>, Void> settingsUpdater) {
      super.applyToAllUnaryMethods(unaryMethodSettingsBuilders, settingsUpdater);
      return this;
    }

    public ImmutableList<UnaryCallSettings.Builder<?, ?>> unaryMethodSettingsBuilders() {
      return unaryMethodSettingsBuilders;
    }

    /** Returns the builder for the settings used for calls to generateContent. */
    public UnaryCallSettings.Builder<GenerateContentRequest, GenerateContentResponse>
        generateContentSettings() {
      return generateContentSettings;
    }

    /** Returns the builder for the settings used for calls to streamGenerateContent. */
    public ServerStreamingCallSettings.Builder<GenerateContentRequest, GenerateContentResponse>
        streamGenerateContentSettings() {
      return streamGenerateContentSettings;
    }

    /** Returns the builder for the settings used for calls to embedContent. */
    public UnaryCallSettings.Builder<EmbedContentRequest, EmbedContentResponse>
        embedContentSettings() {
      return embedContentSettings;
    }

    /** Returns the builder for the settings used for calls to batchEmbedContents. */
    public UnaryCallSettings.Builder<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
        batchEmbedContentsSettings() {
      return batchEmbedContentsSettings;
    }

    /** Returns the builder for the settings used for calls to countTokens. */
    public UnaryCallSettings.Builder<CountTokensRequest, CountTokensResponse>
        countTokensSettings() {
      return countTokensSettings;
    }

    @Override
    public GenerativeServiceStubSettings build() throws IOException {
      return new GenerativeServiceStubSettings(this);
    }
  }
}
