/*
 * Copyright 2023 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.ai.generativelanguage.v1.stub;

import com.google.ai.generativelanguage.v1.BatchEmbedContentsRequest;
import com.google.ai.generativelanguage.v1.BatchEmbedContentsResponse;
import com.google.ai.generativelanguage.v1.CountTokensRequest;
import com.google.ai.generativelanguage.v1.CountTokensResponse;
import com.google.ai.generativelanguage.v1.EmbedContentRequest;
import com.google.ai.generativelanguage.v1.EmbedContentResponse;
import com.google.ai.generativelanguage.v1.GenerateContentRequest;
import com.google.ai.generativelanguage.v1.GenerateContentResponse;
import com.google.api.gax.core.BackgroundResource;
import com.google.api.gax.core.BackgroundResourceAggregation;
import com.google.api.gax.grpc.GrpcCallSettings;
import com.google.api.gax.grpc.GrpcStubCallableFactory;
import com.google.api.gax.rpc.ClientContext;
import com.google.api.gax.rpc.RequestParamsBuilder;
import com.google.api.gax.rpc.ServerStreamingCallable;
import com.google.api.gax.rpc.UnaryCallable;
import com.google.longrunning.stub.GrpcOperationsStub;
import io.grpc.MethodDescriptor;
import io.grpc.protobuf.ProtoUtils;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import javax.annotation.Generated;

// AUTO-GENERATED DOCUMENTATION AND CLASS.
/**
 * gRPC stub implementation for the GenerativeService service API.
 *
 * <p>This class is for advanced usage and reflects the underlying API directly.
 */
@Generated("by gapic-generator-java")
public class GrpcGenerativeServiceStub extends GenerativeServiceStub {
  private static final MethodDescriptor<GenerateContentRequest, GenerateContentResponse>
      generateContentMethodDescriptor =
          MethodDescriptor.<GenerateContentRequest, GenerateContentResponse>newBuilder()
              .setType(MethodDescriptor.MethodType.UNARY)
              .setFullMethodName(
                  "google.ai.generativelanguage.v1.GenerativeService/GenerateContent")
              .setRequestMarshaller(
                  ProtoUtils.marshaller(GenerateContentRequest.getDefaultInstance()))
              .setResponseMarshaller(
                  ProtoUtils.marshaller(GenerateContentResponse.getDefaultInstance()))
              .build();

  private static final MethodDescriptor<GenerateContentRequest, GenerateContentResponse>
      streamGenerateContentMethodDescriptor =
          MethodDescriptor.<GenerateContentRequest, GenerateContentResponse>newBuilder()
              .setType(MethodDescriptor.MethodType.SERVER_STREAMING)
              .setFullMethodName(
                  "google.ai.generativelanguage.v1.GenerativeService/StreamGenerateContent")
              .setRequestMarshaller(
                  ProtoUtils.marshaller(GenerateContentRequest.getDefaultInstance()))
              .setResponseMarshaller(
                  ProtoUtils.marshaller(GenerateContentResponse.getDefaultInstance()))
              .build();

  private static final MethodDescriptor<EmbedContentRequest, EmbedContentResponse>
      embedContentMethodDescriptor =
          MethodDescriptor.<EmbedContentRequest, EmbedContentResponse>newBuilder()
              .setType(MethodDescriptor.MethodType.UNARY)
              .setFullMethodName("google.ai.generativelanguage.v1.GenerativeService/EmbedContent")
              .setRequestMarshaller(ProtoUtils.marshaller(EmbedContentRequest.getDefaultInstance()))
              .setResponseMarshaller(
                  ProtoUtils.marshaller(EmbedContentResponse.getDefaultInstance()))
              .build();

  private static final MethodDescriptor<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
      batchEmbedContentsMethodDescriptor =
          MethodDescriptor.<BatchEmbedContentsRequest, BatchEmbedContentsResponse>newBuilder()
              .setType(MethodDescriptor.MethodType.UNARY)
              .setFullMethodName(
                  "google.ai.generativelanguage.v1.GenerativeService/BatchEmbedContents")
              .setRequestMarshaller(
                  ProtoUtils.marshaller(BatchEmbedContentsRequest.getDefaultInstance()))
              .setResponseMarshaller(
                  ProtoUtils.marshaller(BatchEmbedContentsResponse.getDefaultInstance()))
              .build();

  private static final MethodDescriptor<CountTokensRequest, CountTokensResponse>
      countTokensMethodDescriptor =
          MethodDescriptor.<CountTokensRequest, CountTokensResponse>newBuilder()
              .setType(MethodDescriptor.MethodType.UNARY)
              .setFullMethodName("google.ai.generativelanguage.v1.GenerativeService/CountTokens")
              .setRequestMarshaller(ProtoUtils.marshaller(CountTokensRequest.getDefaultInstance()))
              .setResponseMarshaller(
                  ProtoUtils.marshaller(CountTokensResponse.getDefaultInstance()))
              .build();

  private final UnaryCallable<GenerateContentRequest, GenerateContentResponse>
      generateContentCallable;
  private final ServerStreamingCallable<GenerateContentRequest, GenerateContentResponse>
      streamGenerateContentCallable;
  private final UnaryCallable<EmbedContentRequest, EmbedContentResponse> embedContentCallable;
  private final UnaryCallable<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
      batchEmbedContentsCallable;
  private final UnaryCallable<CountTokensRequest, CountTokensResponse> countTokensCallable;

  private final BackgroundResource backgroundResources;
  private final GrpcOperationsStub operationsStub;
  private final GrpcStubCallableFactory callableFactory;

  public static final GrpcGenerativeServiceStub create(GenerativeServiceStubSettings settings)
      throws IOException {
    return new GrpcGenerativeServiceStub(settings, ClientContext.create(settings));
  }

  public static final GrpcGenerativeServiceStub create(ClientContext clientContext)
      throws IOException {
    return new GrpcGenerativeServiceStub(
        GenerativeServiceStubSettings.newBuilder().build(), clientContext);
  }

  public static final GrpcGenerativeServiceStub create(
      ClientContext clientContext, GrpcStubCallableFactory callableFactory) throws IOException {
    return new GrpcGenerativeServiceStub(
        GenerativeServiceStubSettings.newBuilder().build(), clientContext, callableFactory);
  }

  /**
   * Constructs an instance of GrpcGenerativeServiceStub, using the given settings. This is
   * protected so that it is easy to make a subclass, but otherwise, the static factory methods
   * should be preferred.
   */
  protected GrpcGenerativeServiceStub(
      GenerativeServiceStubSettings settings, ClientContext clientContext) throws IOException {
    this(settings, clientContext, new GrpcGenerativeServiceCallableFactory());
  }

  /**
   * Constructs an instance of GrpcGenerativeServiceStub, using the given settings. This is
   * protected so that it is easy to make a subclass, but otherwise, the static factory methods
   * should be preferred.
   */
  protected GrpcGenerativeServiceStub(
      GenerativeServiceStubSettings settings,
      ClientContext clientContext,
      GrpcStubCallableFactory callableFactory)
      throws IOException {
    this.callableFactory = callableFactory;
    this.operationsStub = GrpcOperationsStub.create(clientContext, callableFactory);

    GrpcCallSettings<GenerateContentRequest, GenerateContentResponse>
        generateContentTransportSettings =
            GrpcCallSettings.<GenerateContentRequest, GenerateContentResponse>newBuilder()
                .setMethodDescriptor(generateContentMethodDescriptor)
                .setParamsExtractor(
                    request -> {
                      RequestParamsBuilder builder = RequestParamsBuilder.create();
                      builder.add("model", String.valueOf(request.getModel()));
                      return builder.build();
                    })
                .build();
    GrpcCallSettings<GenerateContentRequest, GenerateContentResponse>
        streamGenerateContentTransportSettings =
            GrpcCallSettings.<GenerateContentRequest, GenerateContentResponse>newBuilder()
                .setMethodDescriptor(streamGenerateContentMethodDescriptor)
                .setParamsExtractor(
                    request -> {
                      RequestParamsBuilder builder = RequestParamsBuilder.create();
                      builder.add("model", String.valueOf(request.getModel()));
                      return builder.build();
                    })
                .build();
    GrpcCallSettings<EmbedContentRequest, EmbedContentResponse> embedContentTransportSettings =
        GrpcCallSettings.<EmbedContentRequest, EmbedContentResponse>newBuilder()
            .setMethodDescriptor(embedContentMethodDescriptor)
            .setParamsExtractor(
                request -> {
                  RequestParamsBuilder builder = RequestParamsBuilder.create();
                  builder.add("model", String.valueOf(request.getModel()));
                  return builder.build();
                })
            .build();
    GrpcCallSettings<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
        batchEmbedContentsTransportSettings =
            GrpcCallSettings.<BatchEmbedContentsRequest, BatchEmbedContentsResponse>newBuilder()
                .setMethodDescriptor(batchEmbedContentsMethodDescriptor)
                .setParamsExtractor(
                    request -> {
                      RequestParamsBuilder builder = RequestParamsBuilder.create();
                      builder.add("model", String.valueOf(request.getModel()));
                      return builder.build();
                    })
                .build();
    GrpcCallSettings<CountTokensRequest, CountTokensResponse> countTokensTransportSettings =
        GrpcCallSettings.<CountTokensRequest, CountTokensResponse>newBuilder()
            .setMethodDescriptor(countTokensMethodDescriptor)
            .setParamsExtractor(
                request -> {
                  RequestParamsBuilder builder = RequestParamsBuilder.create();
                  builder.add("model", String.valueOf(request.getModel()));
                  return builder.build();
                })
            .build();

    this.generateContentCallable =
        callableFactory.createUnaryCallable(
            generateContentTransportSettings, settings.generateContentSettings(), clientContext);
    this.streamGenerateContentCallable =
        callableFactory.createServerStreamingCallable(
            streamGenerateContentTransportSettings,
            settings.streamGenerateContentSettings(),
            clientContext);
    this.embedContentCallable =
        callableFactory.createUnaryCallable(
            embedContentTransportSettings, settings.embedContentSettings(), clientContext);
    this.batchEmbedContentsCallable =
        callableFactory.createUnaryCallable(
            batchEmbedContentsTransportSettings,
            settings.batchEmbedContentsSettings(),
            clientContext);
    this.countTokensCallable =
        callableFactory.createUnaryCallable(
            countTokensTransportSettings, settings.countTokensSettings(), clientContext);

    this.backgroundResources =
        new BackgroundResourceAggregation(clientContext.getBackgroundResources());
  }

  public GrpcOperationsStub getOperationsStub() {
    return operationsStub;
  }

  @Override
  public UnaryCallable<GenerateContentRequest, GenerateContentResponse> generateContentCallable() {
    return generateContentCallable;
  }

  @Override
  public ServerStreamingCallable<GenerateContentRequest, GenerateContentResponse>
      streamGenerateContentCallable() {
    return streamGenerateContentCallable;
  }

  @Override
  public UnaryCallable<EmbedContentRequest, EmbedContentResponse> embedContentCallable() {
    return embedContentCallable;
  }

  @Override
  public UnaryCallable<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
      batchEmbedContentsCallable() {
    return batchEmbedContentsCallable;
  }

  @Override
  public UnaryCallable<CountTokensRequest, CountTokensResponse> countTokensCallable() {
    return countTokensCallable;
  }

  @Override
  public final void close() {
    try {
      backgroundResources.close();
    } catch (RuntimeException e) {
      throw e;
    } catch (Exception e) {
      throw new IllegalStateException("Failed to close resource", e);
    }
  }

  @Override
  public void shutdown() {
    backgroundResources.shutdown();
  }

  @Override
  public boolean isShutdown() {
    return backgroundResources.isShutdown();
  }

  @Override
  public boolean isTerminated() {
    return backgroundResources.isTerminated();
  }

  @Override
  public void shutdownNow() {
    backgroundResources.shutdownNow();
  }

  @Override
  public boolean awaitTermination(long duration, TimeUnit unit) throws InterruptedException {
    return backgroundResources.awaitTermination(duration, unit);
  }
}
