/*
 * Copyright 2023 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.ai.generativelanguage.v1.stub;

import com.google.ai.generativelanguage.v1.BatchEmbedContentsRequest;
import com.google.ai.generativelanguage.v1.BatchEmbedContentsResponse;
import com.google.ai.generativelanguage.v1.CountTokensRequest;
import com.google.ai.generativelanguage.v1.CountTokensResponse;
import com.google.ai.generativelanguage.v1.EmbedContentRequest;
import com.google.ai.generativelanguage.v1.EmbedContentResponse;
import com.google.ai.generativelanguage.v1.GenerateContentRequest;
import com.google.ai.generativelanguage.v1.GenerateContentResponse;
import com.google.api.core.BetaApi;
import com.google.api.core.InternalApi;
import com.google.api.gax.core.BackgroundResource;
import com.google.api.gax.core.BackgroundResourceAggregation;
import com.google.api.gax.httpjson.ApiMethodDescriptor;
import com.google.api.gax.httpjson.HttpJsonCallSettings;
import com.google.api.gax.httpjson.HttpJsonStubCallableFactory;
import com.google.api.gax.httpjson.ProtoMessageRequestFormatter;
import com.google.api.gax.httpjson.ProtoMessageResponseParser;
import com.google.api.gax.httpjson.ProtoRestSerializer;
import com.google.api.gax.rpc.ClientContext;
import com.google.api.gax.rpc.RequestParamsBuilder;
import com.google.api.gax.rpc.ServerStreamingCallable;
import com.google.api.gax.rpc.UnaryCallable;
import com.google.protobuf.TypeRegistry;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import javax.annotation.Generated;

// AUTO-GENERATED DOCUMENTATION AND CLASS.
/**
 * REST stub implementation for the GenerativeService service API.
 *
 * <p>This class is for advanced usage and reflects the underlying API directly.
 */
@Generated("by gapic-generator-java")
@BetaApi
public class HttpJsonGenerativeServiceStub extends GenerativeServiceStub {
  private static final TypeRegistry typeRegistry = TypeRegistry.newBuilder().build();

  private static final ApiMethodDescriptor<GenerateContentRequest, GenerateContentResponse>
      generateContentMethodDescriptor =
          ApiMethodDescriptor.<GenerateContentRequest, GenerateContentResponse>newBuilder()
              .setFullMethodName(
                  "google.ai.generativelanguage.v1.GenerativeService/GenerateContent")
              .setHttpMethod("POST")
              .setType(ApiMethodDescriptor.MethodType.UNARY)
              .setRequestFormatter(
                  ProtoMessageRequestFormatter.<GenerateContentRequest>newBuilder()
                      .setPath(
                          "/v1/{model=models/*}:generateContent",
                          request -> {
                            Map<String, String> fields = new HashMap<>();
                            ProtoRestSerializer<GenerateContentRequest> serializer =
                                ProtoRestSerializer.create();
                            serializer.putPathParam(fields, "model", request.getModel());
                            return fields;
                          })
                      .setQueryParamsExtractor(
                          request -> {
                            Map<String, List<String>> fields = new HashMap<>();
                            ProtoRestSerializer<GenerateContentRequest> serializer =
                                ProtoRestSerializer.create();
                            serializer.putQueryParam(fields, "$alt", "json;enum-encoding=int");
                            return fields;
                          })
                      .setRequestBodyExtractor(
                          request ->
                              ProtoRestSerializer.create()
                                  .toBody("*", request.toBuilder().clearModel().build(), true))
                      .build())
              .setResponseParser(
                  ProtoMessageResponseParser.<GenerateContentResponse>newBuilder()
                      .setDefaultInstance(GenerateContentResponse.getDefaultInstance())
                      .setDefaultTypeRegistry(typeRegistry)
                      .build())
              .build();

  private static final ApiMethodDescriptor<GenerateContentRequest, GenerateContentResponse>
      streamGenerateContentMethodDescriptor =
          ApiMethodDescriptor.<GenerateContentRequest, GenerateContentResponse>newBuilder()
              .setFullMethodName(
                  "google.ai.generativelanguage.v1.GenerativeService/StreamGenerateContent")
              .setHttpMethod("POST")
              .setType(ApiMethodDescriptor.MethodType.SERVER_STREAMING)
              .setRequestFormatter(
                  ProtoMessageRequestFormatter.<GenerateContentRequest>newBuilder()
                      .setPath(
                          "/v1/{model=models/*}:streamGenerateContent",
                          request -> {
                            Map<String, String> fields = new HashMap<>();
                            ProtoRestSerializer<GenerateContentRequest> serializer =
                                ProtoRestSerializer.create();
                            serializer.putPathParam(fields, "model", request.getModel());
                            return fields;
                          })
                      .setQueryParamsExtractor(
                          request -> {
                            Map<String, List<String>> fields = new HashMap<>();
                            ProtoRestSerializer<GenerateContentRequest> serializer =
                                ProtoRestSerializer.create();
                            serializer.putQueryParam(fields, "$alt", "json;enum-encoding=int");
                            return fields;
                          })
                      .setRequestBodyExtractor(
                          request ->
                              ProtoRestSerializer.create()
                                  .toBody("*", request.toBuilder().clearModel().build(), true))
                      .build())
              .setResponseParser(
                  ProtoMessageResponseParser.<GenerateContentResponse>newBuilder()
                      .setDefaultInstance(GenerateContentResponse.getDefaultInstance())
                      .setDefaultTypeRegistry(typeRegistry)
                      .build())
              .build();

  private static final ApiMethodDescriptor<EmbedContentRequest, EmbedContentResponse>
      embedContentMethodDescriptor =
          ApiMethodDescriptor.<EmbedContentRequest, EmbedContentResponse>newBuilder()
              .setFullMethodName("google.ai.generativelanguage.v1.GenerativeService/EmbedContent")
              .setHttpMethod("POST")
              .setType(ApiMethodDescriptor.MethodType.UNARY)
              .setRequestFormatter(
                  ProtoMessageRequestFormatter.<EmbedContentRequest>newBuilder()
                      .setPath(
                          "/v1/{model=models/*}:embedContent",
                          request -> {
                            Map<String, String> fields = new HashMap<>();
                            ProtoRestSerializer<EmbedContentRequest> serializer =
                                ProtoRestSerializer.create();
                            serializer.putPathParam(fields, "model", request.getModel());
                            return fields;
                          })
                      .setQueryParamsExtractor(
                          request -> {
                            Map<String, List<String>> fields = new HashMap<>();
                            ProtoRestSerializer<EmbedContentRequest> serializer =
                                ProtoRestSerializer.create();
                            serializer.putQueryParam(fields, "$alt", "json;enum-encoding=int");
                            return fields;
                          })
                      .setRequestBodyExtractor(
                          request ->
                              ProtoRestSerializer.create()
                                  .toBody("*", request.toBuilder().clearModel().build(), true))
                      .build())
              .setResponseParser(
                  ProtoMessageResponseParser.<EmbedContentResponse>newBuilder()
                      .setDefaultInstance(EmbedContentResponse.getDefaultInstance())
                      .setDefaultTypeRegistry(typeRegistry)
                      .build())
              .build();

  private static final ApiMethodDescriptor<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
      batchEmbedContentsMethodDescriptor =
          ApiMethodDescriptor.<BatchEmbedContentsRequest, BatchEmbedContentsResponse>newBuilder()
              .setFullMethodName(
                  "google.ai.generativelanguage.v1.GenerativeService/BatchEmbedContents")
              .setHttpMethod("POST")
              .setType(ApiMethodDescriptor.MethodType.UNARY)
              .setRequestFormatter(
                  ProtoMessageRequestFormatter.<BatchEmbedContentsRequest>newBuilder()
                      .setPath(
                          "/v1/{model=models/*}:batchEmbedContents",
                          request -> {
                            Map<String, String> fields = new HashMap<>();
                            ProtoRestSerializer<BatchEmbedContentsRequest> serializer =
                                ProtoRestSerializer.create();
                            serializer.putPathParam(fields, "model", request.getModel());
                            return fields;
                          })
                      .setQueryParamsExtractor(
                          request -> {
                            Map<String, List<String>> fields = new HashMap<>();
                            ProtoRestSerializer<BatchEmbedContentsRequest> serializer =
                                ProtoRestSerializer.create();
                            serializer.putQueryParam(fields, "$alt", "json;enum-encoding=int");
                            return fields;
                          })
                      .setRequestBodyExtractor(
                          request ->
                              ProtoRestSerializer.create()
                                  .toBody("*", request.toBuilder().clearModel().build(), true))
                      .build())
              .setResponseParser(
                  ProtoMessageResponseParser.<BatchEmbedContentsResponse>newBuilder()
                      .setDefaultInstance(BatchEmbedContentsResponse.getDefaultInstance())
                      .setDefaultTypeRegistry(typeRegistry)
                      .build())
              .build();

  private static final ApiMethodDescriptor<CountTokensRequest, CountTokensResponse>
      countTokensMethodDescriptor =
          ApiMethodDescriptor.<CountTokensRequest, CountTokensResponse>newBuilder()
              .setFullMethodName("google.ai.generativelanguage.v1.GenerativeService/CountTokens")
              .setHttpMethod("POST")
              .setType(ApiMethodDescriptor.MethodType.UNARY)
              .setRequestFormatter(
                  ProtoMessageRequestFormatter.<CountTokensRequest>newBuilder()
                      .setPath(
                          "/v1/{model=models/*}:countTokens",
                          request -> {
                            Map<String, String> fields = new HashMap<>();
                            ProtoRestSerializer<CountTokensRequest> serializer =
                                ProtoRestSerializer.create();
                            serializer.putPathParam(fields, "model", request.getModel());
                            return fields;
                          })
                      .setQueryParamsExtractor(
                          request -> {
                            Map<String, List<String>> fields = new HashMap<>();
                            ProtoRestSerializer<CountTokensRequest> serializer =
                                ProtoRestSerializer.create();
                            serializer.putQueryParam(fields, "$alt", "json;enum-encoding=int");
                            return fields;
                          })
                      .setRequestBodyExtractor(
                          request ->
                              ProtoRestSerializer.create()
                                  .toBody("*", request.toBuilder().clearModel().build(), true))
                      .build())
              .setResponseParser(
                  ProtoMessageResponseParser.<CountTokensResponse>newBuilder()
                      .setDefaultInstance(CountTokensResponse.getDefaultInstance())
                      .setDefaultTypeRegistry(typeRegistry)
                      .build())
              .build();

  private final UnaryCallable<GenerateContentRequest, GenerateContentResponse>
      generateContentCallable;
  private final ServerStreamingCallable<GenerateContentRequest, GenerateContentResponse>
      streamGenerateContentCallable;
  private final UnaryCallable<EmbedContentRequest, EmbedContentResponse> embedContentCallable;
  private final UnaryCallable<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
      batchEmbedContentsCallable;
  private final UnaryCallable<CountTokensRequest, CountTokensResponse> countTokensCallable;

  private final BackgroundResource backgroundResources;
  private final HttpJsonStubCallableFactory callableFactory;

  public static final HttpJsonGenerativeServiceStub create(GenerativeServiceStubSettings settings)
      throws IOException {
    return new HttpJsonGenerativeServiceStub(settings, ClientContext.create(settings));
  }

  public static final HttpJsonGenerativeServiceStub create(ClientContext clientContext)
      throws IOException {
    return new HttpJsonGenerativeServiceStub(
        GenerativeServiceStubSettings.newHttpJsonBuilder().build(), clientContext);
  }

  public static final HttpJsonGenerativeServiceStub create(
      ClientContext clientContext, HttpJsonStubCallableFactory callableFactory) throws IOException {
    return new HttpJsonGenerativeServiceStub(
        GenerativeServiceStubSettings.newHttpJsonBuilder().build(), clientContext, callableFactory);
  }

  /**
   * Constructs an instance of HttpJsonGenerativeServiceStub, using the given settings. This is
   * protected so that it is easy to make a subclass, but otherwise, the static factory methods
   * should be preferred.
   */
  protected HttpJsonGenerativeServiceStub(
      GenerativeServiceStubSettings settings, ClientContext clientContext) throws IOException {
    this(settings, clientContext, new HttpJsonGenerativeServiceCallableFactory());
  }

  /**
   * Constructs an instance of HttpJsonGenerativeServiceStub, using the given settings. This is
   * protected so that it is easy to make a subclass, but otherwise, the static factory methods
   * should be preferred.
   */
  protected HttpJsonGenerativeServiceStub(
      GenerativeServiceStubSettings settings,
      ClientContext clientContext,
      HttpJsonStubCallableFactory callableFactory)
      throws IOException {
    this.callableFactory = callableFactory;

    HttpJsonCallSettings<GenerateContentRequest, GenerateContentResponse>
        generateContentTransportSettings =
            HttpJsonCallSettings.<GenerateContentRequest, GenerateContentResponse>newBuilder()
                .setMethodDescriptor(generateContentMethodDescriptor)
                .setTypeRegistry(typeRegistry)
                .setParamsExtractor(
                    request -> {
                      RequestParamsBuilder builder = RequestParamsBuilder.create();
                      builder.add("model", String.valueOf(request.getModel()));
                      return builder.build();
                    })
                .build();
    HttpJsonCallSettings<GenerateContentRequest, GenerateContentResponse>
        streamGenerateContentTransportSettings =
            HttpJsonCallSettings.<GenerateContentRequest, GenerateContentResponse>newBuilder()
                .setMethodDescriptor(streamGenerateContentMethodDescriptor)
                .setTypeRegistry(typeRegistry)
                .setParamsExtractor(
                    request -> {
                      RequestParamsBuilder builder = RequestParamsBuilder.create();
                      builder.add("model", String.valueOf(request.getModel()));
                      return builder.build();
                    })
                .build();
    HttpJsonCallSettings<EmbedContentRequest, EmbedContentResponse> embedContentTransportSettings =
        HttpJsonCallSettings.<EmbedContentRequest, EmbedContentResponse>newBuilder()
            .setMethodDescriptor(embedContentMethodDescriptor)
            .setTypeRegistry(typeRegistry)
            .setParamsExtractor(
                request -> {
                  RequestParamsBuilder builder = RequestParamsBuilder.create();
                  builder.add("model", String.valueOf(request.getModel()));
                  return builder.build();
                })
            .build();
    HttpJsonCallSettings<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
        batchEmbedContentsTransportSettings =
            HttpJsonCallSettings.<BatchEmbedContentsRequest, BatchEmbedContentsResponse>newBuilder()
                .setMethodDescriptor(batchEmbedContentsMethodDescriptor)
                .setTypeRegistry(typeRegistry)
                .setParamsExtractor(
                    request -> {
                      RequestParamsBuilder builder = RequestParamsBuilder.create();
                      builder.add("model", String.valueOf(request.getModel()));
                      return builder.build();
                    })
                .build();
    HttpJsonCallSettings<CountTokensRequest, CountTokensResponse> countTokensTransportSettings =
        HttpJsonCallSettings.<CountTokensRequest, CountTokensResponse>newBuilder()
            .setMethodDescriptor(countTokensMethodDescriptor)
            .setTypeRegistry(typeRegistry)
            .setParamsExtractor(
                request -> {
                  RequestParamsBuilder builder = RequestParamsBuilder.create();
                  builder.add("model", String.valueOf(request.getModel()));
                  return builder.build();
                })
            .build();

    this.generateContentCallable =
        callableFactory.createUnaryCallable(
            generateContentTransportSettings, settings.generateContentSettings(), clientContext);
    this.streamGenerateContentCallable =
        callableFactory.createServerStreamingCallable(
            streamGenerateContentTransportSettings,
            settings.streamGenerateContentSettings(),
            clientContext);
    this.embedContentCallable =
        callableFactory.createUnaryCallable(
            embedContentTransportSettings, settings.embedContentSettings(), clientContext);
    this.batchEmbedContentsCallable =
        callableFactory.createUnaryCallable(
            batchEmbedContentsTransportSettings,
            settings.batchEmbedContentsSettings(),
            clientContext);
    this.countTokensCallable =
        callableFactory.createUnaryCallable(
            countTokensTransportSettings, settings.countTokensSettings(), clientContext);

    this.backgroundResources =
        new BackgroundResourceAggregation(clientContext.getBackgroundResources());
  }

  @InternalApi
  public static List<ApiMethodDescriptor> getMethodDescriptors() {
    List<ApiMethodDescriptor> methodDescriptors = new ArrayList<>();
    methodDescriptors.add(generateContentMethodDescriptor);
    methodDescriptors.add(streamGenerateContentMethodDescriptor);
    methodDescriptors.add(embedContentMethodDescriptor);
    methodDescriptors.add(batchEmbedContentsMethodDescriptor);
    methodDescriptors.add(countTokensMethodDescriptor);
    return methodDescriptors;
  }

  @Override
  public UnaryCallable<GenerateContentRequest, GenerateContentResponse> generateContentCallable() {
    return generateContentCallable;
  }

  @Override
  public ServerStreamingCallable<GenerateContentRequest, GenerateContentResponse>
      streamGenerateContentCallable() {
    return streamGenerateContentCallable;
  }

  @Override
  public UnaryCallable<EmbedContentRequest, EmbedContentResponse> embedContentCallable() {
    return embedContentCallable;
  }

  @Override
  public UnaryCallable<BatchEmbedContentsRequest, BatchEmbedContentsResponse>
      batchEmbedContentsCallable() {
    return batchEmbedContentsCallable;
  }

  @Override
  public UnaryCallable<CountTokensRequest, CountTokensResponse> countTokensCallable() {
    return countTokensCallable;
  }

  @Override
  public final void close() {
    try {
      backgroundResources.close();
    } catch (RuntimeException e) {
      throw e;
    } catch (Exception e) {
      throw new IllegalStateException("Failed to close resource", e);
    }
  }

  @Override
  public void shutdown() {
    backgroundResources.shutdown();
  }

  @Override
  public boolean isShutdown() {
    return backgroundResources.isShutdown();
  }

  @Override
  public boolean isTerminated() {
    return backgroundResources.isTerminated();
  }

  @Override
  public void shutdownNow() {
    backgroundResources.shutdownNow();
  }

  @Override
  public boolean awaitTermination(long duration, TimeUnit unit) throws InterruptedException {
    return backgroundResources.awaitTermination(duration, unit);
  }
}
