/*
 * Copyright 2023 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.ai.generativelanguage.v1;

import com.google.ai.generativelanguage.v1.stub.HttpJsonGenerativeServiceStub;
import com.google.api.gax.core.NoCredentialsProvider;
import com.google.api.gax.httpjson.GaxHttpJsonProperties;
import com.google.api.gax.httpjson.testing.MockHttpService;
import com.google.api.gax.rpc.ApiClientHeaderProvider;
import com.google.api.gax.rpc.ApiException;
import com.google.api.gax.rpc.ApiExceptionFactory;
import com.google.api.gax.rpc.InvalidArgumentException;
import com.google.api.gax.rpc.StatusCode;
import com.google.api.gax.rpc.testing.FakeStatusCode;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.Generated;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

@Generated("by gapic-generator-java")
public class GenerativeServiceClientHttpJsonTest {
  private static MockHttpService mockService;
  private static GenerativeServiceClient client;

  @BeforeClass
  public static void startStaticServer() throws IOException {
    mockService =
        new MockHttpService(
            HttpJsonGenerativeServiceStub.getMethodDescriptors(),
            GenerativeServiceSettings.getDefaultEndpoint());
    GenerativeServiceSettings settings =
        GenerativeServiceSettings.newHttpJsonBuilder()
            .setTransportChannelProvider(
                GenerativeServiceSettings.defaultHttpJsonTransportProviderBuilder()
                    .setHttpTransport(mockService)
                    .build())
            .setCredentialsProvider(NoCredentialsProvider.create())
            .build();
    client = GenerativeServiceClient.create(settings);
  }

  @AfterClass
  public static void stopServer() {
    client.close();
  }

  @Before
  public void setUp() {}

  @After
  public void tearDown() throws Exception {
    mockService.reset();
  }

  @Test
  public void generateContentTest() throws Exception {
    GenerateContentResponse expectedResponse =
        GenerateContentResponse.newBuilder()
            .addAllCandidates(new ArrayList<Candidate>())
            .setPromptFeedback(GenerateContentResponse.PromptFeedback.newBuilder().build())
            .build();
    mockService.addResponse(expectedResponse);

    ModelName model = ModelName.of("[MODEL]");
    List<Content> contents = new ArrayList<>();

    GenerateContentResponse actualResponse = client.generateContent(model, contents);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<String> actualRequests = mockService.getRequestPaths();
    Assert.assertEquals(1, actualRequests.size());

    String apiClientHeaderKey =
        mockService
            .getRequestHeaders()
            .get(ApiClientHeaderProvider.getDefaultApiClientHeaderKey())
            .iterator()
            .next();
    Assert.assertTrue(
        GaxHttpJsonProperties.getDefaultApiClientHeaderPattern()
            .matcher(apiClientHeaderKey)
            .matches());
  }

  @Test
  public void generateContentExceptionTest() throws Exception {
    ApiException exception =
        ApiExceptionFactory.createException(
            new Exception(), FakeStatusCode.of(StatusCode.Code.INVALID_ARGUMENT), false);
    mockService.addException(exception);

    try {
      ModelName model = ModelName.of("[MODEL]");
      List<Content> contents = new ArrayList<>();
      client.generateContent(model, contents);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void generateContentTest2() throws Exception {
    GenerateContentResponse expectedResponse =
        GenerateContentResponse.newBuilder()
            .addAllCandidates(new ArrayList<Candidate>())
            .setPromptFeedback(GenerateContentResponse.PromptFeedback.newBuilder().build())
            .build();
    mockService.addResponse(expectedResponse);

    String model = "models/model-8306";
    List<Content> contents = new ArrayList<>();

    GenerateContentResponse actualResponse = client.generateContent(model, contents);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<String> actualRequests = mockService.getRequestPaths();
    Assert.assertEquals(1, actualRequests.size());

    String apiClientHeaderKey =
        mockService
            .getRequestHeaders()
            .get(ApiClientHeaderProvider.getDefaultApiClientHeaderKey())
            .iterator()
            .next();
    Assert.assertTrue(
        GaxHttpJsonProperties.getDefaultApiClientHeaderPattern()
            .matcher(apiClientHeaderKey)
            .matches());
  }

  @Test
  public void generateContentExceptionTest2() throws Exception {
    ApiException exception =
        ApiExceptionFactory.createException(
            new Exception(), FakeStatusCode.of(StatusCode.Code.INVALID_ARGUMENT), false);
    mockService.addException(exception);

    try {
      String model = "models/model-8306";
      List<Content> contents = new ArrayList<>();
      client.generateContent(model, contents);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void streamGenerateContentTest() throws Exception {}

  @Test
  public void streamGenerateContentExceptionTest() throws Exception {
    ApiException exception =
        ApiExceptionFactory.createException(
            new Exception(), FakeStatusCode.of(StatusCode.Code.INVALID_ARGUMENT), false);
    mockService.addException(exception);
  }

  @Test
  public void embedContentTest() throws Exception {
    EmbedContentResponse expectedResponse =
        EmbedContentResponse.newBuilder()
            .setEmbedding(ContentEmbedding.newBuilder().build())
            .build();
    mockService.addResponse(expectedResponse);

    ModelName model = ModelName.of("[MODEL]");
    Content content = Content.newBuilder().build();

    EmbedContentResponse actualResponse = client.embedContent(model, content);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<String> actualRequests = mockService.getRequestPaths();
    Assert.assertEquals(1, actualRequests.size());

    String apiClientHeaderKey =
        mockService
            .getRequestHeaders()
            .get(ApiClientHeaderProvider.getDefaultApiClientHeaderKey())
            .iterator()
            .next();
    Assert.assertTrue(
        GaxHttpJsonProperties.getDefaultApiClientHeaderPattern()
            .matcher(apiClientHeaderKey)
            .matches());
  }

  @Test
  public void embedContentExceptionTest() throws Exception {
    ApiException exception =
        ApiExceptionFactory.createException(
            new Exception(), FakeStatusCode.of(StatusCode.Code.INVALID_ARGUMENT), false);
    mockService.addException(exception);

    try {
      ModelName model = ModelName.of("[MODEL]");
      Content content = Content.newBuilder().build();
      client.embedContent(model, content);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void embedContentTest2() throws Exception {
    EmbedContentResponse expectedResponse =
        EmbedContentResponse.newBuilder()
            .setEmbedding(ContentEmbedding.newBuilder().build())
            .build();
    mockService.addResponse(expectedResponse);

    String model = "models/model-8306";
    Content content = Content.newBuilder().build();

    EmbedContentResponse actualResponse = client.embedContent(model, content);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<String> actualRequests = mockService.getRequestPaths();
    Assert.assertEquals(1, actualRequests.size());

    String apiClientHeaderKey =
        mockService
            .getRequestHeaders()
            .get(ApiClientHeaderProvider.getDefaultApiClientHeaderKey())
            .iterator()
            .next();
    Assert.assertTrue(
        GaxHttpJsonProperties.getDefaultApiClientHeaderPattern()
            .matcher(apiClientHeaderKey)
            .matches());
  }

  @Test
  public void embedContentExceptionTest2() throws Exception {
    ApiException exception =
        ApiExceptionFactory.createException(
            new Exception(), FakeStatusCode.of(StatusCode.Code.INVALID_ARGUMENT), false);
    mockService.addException(exception);

    try {
      String model = "models/model-8306";
      Content content = Content.newBuilder().build();
      client.embedContent(model, content);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void batchEmbedContentsTest() throws Exception {
    BatchEmbedContentsResponse expectedResponse =
        BatchEmbedContentsResponse.newBuilder()
            .addAllEmbeddings(new ArrayList<ContentEmbedding>())
            .build();
    mockService.addResponse(expectedResponse);

    ModelName model = ModelName.of("[MODEL]");
    List<EmbedContentRequest> requests = new ArrayList<>();

    BatchEmbedContentsResponse actualResponse = client.batchEmbedContents(model, requests);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<String> actualRequests = mockService.getRequestPaths();
    Assert.assertEquals(1, actualRequests.size());

    String apiClientHeaderKey =
        mockService
            .getRequestHeaders()
            .get(ApiClientHeaderProvider.getDefaultApiClientHeaderKey())
            .iterator()
            .next();
    Assert.assertTrue(
        GaxHttpJsonProperties.getDefaultApiClientHeaderPattern()
            .matcher(apiClientHeaderKey)
            .matches());
  }

  @Test
  public void batchEmbedContentsExceptionTest() throws Exception {
    ApiException exception =
        ApiExceptionFactory.createException(
            new Exception(), FakeStatusCode.of(StatusCode.Code.INVALID_ARGUMENT), false);
    mockService.addException(exception);

    try {
      ModelName model = ModelName.of("[MODEL]");
      List<EmbedContentRequest> requests = new ArrayList<>();
      client.batchEmbedContents(model, requests);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void batchEmbedContentsTest2() throws Exception {
    BatchEmbedContentsResponse expectedResponse =
        BatchEmbedContentsResponse.newBuilder()
            .addAllEmbeddings(new ArrayList<ContentEmbedding>())
            .build();
    mockService.addResponse(expectedResponse);

    String model = "models/model-8306";
    List<EmbedContentRequest> requests = new ArrayList<>();

    BatchEmbedContentsResponse actualResponse = client.batchEmbedContents(model, requests);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<String> actualRequests = mockService.getRequestPaths();
    Assert.assertEquals(1, actualRequests.size());

    String apiClientHeaderKey =
        mockService
            .getRequestHeaders()
            .get(ApiClientHeaderProvider.getDefaultApiClientHeaderKey())
            .iterator()
            .next();
    Assert.assertTrue(
        GaxHttpJsonProperties.getDefaultApiClientHeaderPattern()
            .matcher(apiClientHeaderKey)
            .matches());
  }

  @Test
  public void batchEmbedContentsExceptionTest2() throws Exception {
    ApiException exception =
        ApiExceptionFactory.createException(
            new Exception(), FakeStatusCode.of(StatusCode.Code.INVALID_ARGUMENT), false);
    mockService.addException(exception);

    try {
      String model = "models/model-8306";
      List<EmbedContentRequest> requests = new ArrayList<>();
      client.batchEmbedContents(model, requests);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void countTokensTest() throws Exception {
    CountTokensResponse expectedResponse =
        CountTokensResponse.newBuilder().setTotalTokens(730673909).build();
    mockService.addResponse(expectedResponse);

    ModelName model = ModelName.of("[MODEL]");
    List<Content> contents = new ArrayList<>();

    CountTokensResponse actualResponse = client.countTokens(model, contents);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<String> actualRequests = mockService.getRequestPaths();
    Assert.assertEquals(1, actualRequests.size());

    String apiClientHeaderKey =
        mockService
            .getRequestHeaders()
            .get(ApiClientHeaderProvider.getDefaultApiClientHeaderKey())
            .iterator()
            .next();
    Assert.assertTrue(
        GaxHttpJsonProperties.getDefaultApiClientHeaderPattern()
            .matcher(apiClientHeaderKey)
            .matches());
  }

  @Test
  public void countTokensExceptionTest() throws Exception {
    ApiException exception =
        ApiExceptionFactory.createException(
            new Exception(), FakeStatusCode.of(StatusCode.Code.INVALID_ARGUMENT), false);
    mockService.addException(exception);

    try {
      ModelName model = ModelName.of("[MODEL]");
      List<Content> contents = new ArrayList<>();
      client.countTokens(model, contents);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void countTokensTest2() throws Exception {
    CountTokensResponse expectedResponse =
        CountTokensResponse.newBuilder().setTotalTokens(730673909).build();
    mockService.addResponse(expectedResponse);

    String model = "models/model-8306";
    List<Content> contents = new ArrayList<>();

    CountTokensResponse actualResponse = client.countTokens(model, contents);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<String> actualRequests = mockService.getRequestPaths();
    Assert.assertEquals(1, actualRequests.size());

    String apiClientHeaderKey =
        mockService
            .getRequestHeaders()
            .get(ApiClientHeaderProvider.getDefaultApiClientHeaderKey())
            .iterator()
            .next();
    Assert.assertTrue(
        GaxHttpJsonProperties.getDefaultApiClientHeaderPattern()
            .matcher(apiClientHeaderKey)
            .matches());
  }

  @Test
  public void countTokensExceptionTest2() throws Exception {
    ApiException exception =
        ApiExceptionFactory.createException(
            new Exception(), FakeStatusCode.of(StatusCode.Code.INVALID_ARGUMENT), false);
    mockService.addException(exception);

    try {
      String model = "models/model-8306";
      List<Content> contents = new ArrayList<>();
      client.countTokens(model, contents);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }
}
