/*
 * Copyright 2023 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.ai.generativelanguage.v1;

import com.google.api.gax.core.NoCredentialsProvider;
import com.google.api.gax.grpc.GaxGrpcProperties;
import com.google.api.gax.grpc.testing.LocalChannelProvider;
import com.google.api.gax.grpc.testing.MockGrpcService;
import com.google.api.gax.grpc.testing.MockServiceHelper;
import com.google.api.gax.grpc.testing.MockStreamObserver;
import com.google.api.gax.rpc.ApiClientHeaderProvider;
import com.google.api.gax.rpc.InvalidArgumentException;
import com.google.api.gax.rpc.ServerStreamingCallable;
import com.google.api.gax.rpc.StatusCode;
import com.google.protobuf.AbstractMessage;
import io.grpc.StatusRuntimeException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.ExecutionException;
import javax.annotation.Generated;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

@Generated("by gapic-generator-java")
public class GenerativeServiceClientTest {
  private static MockGenerativeService mockGenerativeService;
  private static MockServiceHelper mockServiceHelper;
  private LocalChannelProvider channelProvider;
  private GenerativeServiceClient client;

  @BeforeClass
  public static void startStaticServer() {
    mockGenerativeService = new MockGenerativeService();
    mockServiceHelper =
        new MockServiceHelper(
            UUID.randomUUID().toString(), Arrays.<MockGrpcService>asList(mockGenerativeService));
    mockServiceHelper.start();
  }

  @AfterClass
  public static void stopServer() {
    mockServiceHelper.stop();
  }

  @Before
  public void setUp() throws IOException {
    mockServiceHelper.reset();
    channelProvider = mockServiceHelper.createChannelProvider();
    GenerativeServiceSettings settings =
        GenerativeServiceSettings.newBuilder()
            .setTransportChannelProvider(channelProvider)
            .setCredentialsProvider(NoCredentialsProvider.create())
            .build();
    client = GenerativeServiceClient.create(settings);
  }

  @After
  public void tearDown() throws Exception {
    client.close();
  }

  @Test
  public void generateContentTest() throws Exception {
    GenerateContentResponse expectedResponse =
        GenerateContentResponse.newBuilder()
            .addAllCandidates(new ArrayList<Candidate>())
            .setPromptFeedback(GenerateContentResponse.PromptFeedback.newBuilder().build())
            .build();
    mockGenerativeService.addResponse(expectedResponse);

    ModelName model = ModelName.of("[MODEL]");
    List<Content> contents = new ArrayList<>();

    GenerateContentResponse actualResponse = client.generateContent(model, contents);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<AbstractMessage> actualRequests = mockGenerativeService.getRequests();
    Assert.assertEquals(1, actualRequests.size());
    GenerateContentRequest actualRequest = ((GenerateContentRequest) actualRequests.get(0));

    Assert.assertEquals(model.toString(), actualRequest.getModel());
    Assert.assertEquals(contents, actualRequest.getContentsList());
    Assert.assertTrue(
        channelProvider.isHeaderSent(
            ApiClientHeaderProvider.getDefaultApiClientHeaderKey(),
            GaxGrpcProperties.getDefaultApiClientHeaderPattern()));
  }

  @Test
  public void generateContentExceptionTest() throws Exception {
    StatusRuntimeException exception = new StatusRuntimeException(io.grpc.Status.INVALID_ARGUMENT);
    mockGenerativeService.addException(exception);

    try {
      ModelName model = ModelName.of("[MODEL]");
      List<Content> contents = new ArrayList<>();
      client.generateContent(model, contents);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void generateContentTest2() throws Exception {
    GenerateContentResponse expectedResponse =
        GenerateContentResponse.newBuilder()
            .addAllCandidates(new ArrayList<Candidate>())
            .setPromptFeedback(GenerateContentResponse.PromptFeedback.newBuilder().build())
            .build();
    mockGenerativeService.addResponse(expectedResponse);

    String model = "model104069929";
    List<Content> contents = new ArrayList<>();

    GenerateContentResponse actualResponse = client.generateContent(model, contents);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<AbstractMessage> actualRequests = mockGenerativeService.getRequests();
    Assert.assertEquals(1, actualRequests.size());
    GenerateContentRequest actualRequest = ((GenerateContentRequest) actualRequests.get(0));

    Assert.assertEquals(model, actualRequest.getModel());
    Assert.assertEquals(contents, actualRequest.getContentsList());
    Assert.assertTrue(
        channelProvider.isHeaderSent(
            ApiClientHeaderProvider.getDefaultApiClientHeaderKey(),
            GaxGrpcProperties.getDefaultApiClientHeaderPattern()));
  }

  @Test
  public void generateContentExceptionTest2() throws Exception {
    StatusRuntimeException exception = new StatusRuntimeException(io.grpc.Status.INVALID_ARGUMENT);
    mockGenerativeService.addException(exception);

    try {
      String model = "model104069929";
      List<Content> contents = new ArrayList<>();
      client.generateContent(model, contents);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void streamGenerateContentTest() throws Exception {
    GenerateContentResponse expectedResponse =
        GenerateContentResponse.newBuilder()
            .addAllCandidates(new ArrayList<Candidate>())
            .setPromptFeedback(GenerateContentResponse.PromptFeedback.newBuilder().build())
            .build();
    mockGenerativeService.addResponse(expectedResponse);
    GenerateContentRequest request =
        GenerateContentRequest.newBuilder()
            .setModel(ModelName.of("[MODEL]").toString())
            .addAllContents(new ArrayList<Content>())
            .addAllSafetySettings(new ArrayList<SafetySetting>())
            .setGenerationConfig(GenerationConfig.newBuilder().build())
            .build();

    MockStreamObserver<GenerateContentResponse> responseObserver = new MockStreamObserver<>();

    ServerStreamingCallable<GenerateContentRequest, GenerateContentResponse> callable =
        client.streamGenerateContentCallable();
    callable.serverStreamingCall(request, responseObserver);

    List<GenerateContentResponse> actualResponses = responseObserver.future().get();
    Assert.assertEquals(1, actualResponses.size());
    Assert.assertEquals(expectedResponse, actualResponses.get(0));
  }

  @Test
  public void streamGenerateContentExceptionTest() throws Exception {
    StatusRuntimeException exception = new StatusRuntimeException(io.grpc.Status.INVALID_ARGUMENT);
    mockGenerativeService.addException(exception);
    GenerateContentRequest request =
        GenerateContentRequest.newBuilder()
            .setModel(ModelName.of("[MODEL]").toString())
            .addAllContents(new ArrayList<Content>())
            .addAllSafetySettings(new ArrayList<SafetySetting>())
            .setGenerationConfig(GenerationConfig.newBuilder().build())
            .build();

    MockStreamObserver<GenerateContentResponse> responseObserver = new MockStreamObserver<>();

    ServerStreamingCallable<GenerateContentRequest, GenerateContentResponse> callable =
        client.streamGenerateContentCallable();
    callable.serverStreamingCall(request, responseObserver);

    try {
      List<GenerateContentResponse> actualResponses = responseObserver.future().get();
      Assert.fail("No exception thrown");
    } catch (ExecutionException e) {
      Assert.assertTrue(e.getCause() instanceof InvalidArgumentException);
      InvalidArgumentException apiException = ((InvalidArgumentException) e.getCause());
      Assert.assertEquals(StatusCode.Code.INVALID_ARGUMENT, apiException.getStatusCode().getCode());
    }
  }

  @Test
  public void embedContentTest() throws Exception {
    EmbedContentResponse expectedResponse =
        EmbedContentResponse.newBuilder()
            .setEmbedding(ContentEmbedding.newBuilder().build())
            .build();
    mockGenerativeService.addResponse(expectedResponse);

    ModelName model = ModelName.of("[MODEL]");
    Content content = Content.newBuilder().build();

    EmbedContentResponse actualResponse = client.embedContent(model, content);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<AbstractMessage> actualRequests = mockGenerativeService.getRequests();
    Assert.assertEquals(1, actualRequests.size());
    EmbedContentRequest actualRequest = ((EmbedContentRequest) actualRequests.get(0));

    Assert.assertEquals(model.toString(), actualRequest.getModel());
    Assert.assertEquals(content, actualRequest.getContent());
    Assert.assertTrue(
        channelProvider.isHeaderSent(
            ApiClientHeaderProvider.getDefaultApiClientHeaderKey(),
            GaxGrpcProperties.getDefaultApiClientHeaderPattern()));
  }

  @Test
  public void embedContentExceptionTest() throws Exception {
    StatusRuntimeException exception = new StatusRuntimeException(io.grpc.Status.INVALID_ARGUMENT);
    mockGenerativeService.addException(exception);

    try {
      ModelName model = ModelName.of("[MODEL]");
      Content content = Content.newBuilder().build();
      client.embedContent(model, content);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void embedContentTest2() throws Exception {
    EmbedContentResponse expectedResponse =
        EmbedContentResponse.newBuilder()
            .setEmbedding(ContentEmbedding.newBuilder().build())
            .build();
    mockGenerativeService.addResponse(expectedResponse);

    String model = "model104069929";
    Content content = Content.newBuilder().build();

    EmbedContentResponse actualResponse = client.embedContent(model, content);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<AbstractMessage> actualRequests = mockGenerativeService.getRequests();
    Assert.assertEquals(1, actualRequests.size());
    EmbedContentRequest actualRequest = ((EmbedContentRequest) actualRequests.get(0));

    Assert.assertEquals(model, actualRequest.getModel());
    Assert.assertEquals(content, actualRequest.getContent());
    Assert.assertTrue(
        channelProvider.isHeaderSent(
            ApiClientHeaderProvider.getDefaultApiClientHeaderKey(),
            GaxGrpcProperties.getDefaultApiClientHeaderPattern()));
  }

  @Test
  public void embedContentExceptionTest2() throws Exception {
    StatusRuntimeException exception = new StatusRuntimeException(io.grpc.Status.INVALID_ARGUMENT);
    mockGenerativeService.addException(exception);

    try {
      String model = "model104069929";
      Content content = Content.newBuilder().build();
      client.embedContent(model, content);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void batchEmbedContentsTest() throws Exception {
    BatchEmbedContentsResponse expectedResponse =
        BatchEmbedContentsResponse.newBuilder()
            .addAllEmbeddings(new ArrayList<ContentEmbedding>())
            .build();
    mockGenerativeService.addResponse(expectedResponse);

    ModelName model = ModelName.of("[MODEL]");
    List<EmbedContentRequest> requests = new ArrayList<>();

    BatchEmbedContentsResponse actualResponse = client.batchEmbedContents(model, requests);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<AbstractMessage> actualRequests = mockGenerativeService.getRequests();
    Assert.assertEquals(1, actualRequests.size());
    BatchEmbedContentsRequest actualRequest = ((BatchEmbedContentsRequest) actualRequests.get(0));

    Assert.assertEquals(model.toString(), actualRequest.getModel());
    Assert.assertEquals(requests, actualRequest.getRequestsList());
    Assert.assertTrue(
        channelProvider.isHeaderSent(
            ApiClientHeaderProvider.getDefaultApiClientHeaderKey(),
            GaxGrpcProperties.getDefaultApiClientHeaderPattern()));
  }

  @Test
  public void batchEmbedContentsExceptionTest() throws Exception {
    StatusRuntimeException exception = new StatusRuntimeException(io.grpc.Status.INVALID_ARGUMENT);
    mockGenerativeService.addException(exception);

    try {
      ModelName model = ModelName.of("[MODEL]");
      List<EmbedContentRequest> requests = new ArrayList<>();
      client.batchEmbedContents(model, requests);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void batchEmbedContentsTest2() throws Exception {
    BatchEmbedContentsResponse expectedResponse =
        BatchEmbedContentsResponse.newBuilder()
            .addAllEmbeddings(new ArrayList<ContentEmbedding>())
            .build();
    mockGenerativeService.addResponse(expectedResponse);

    String model = "model104069929";
    List<EmbedContentRequest> requests = new ArrayList<>();

    BatchEmbedContentsResponse actualResponse = client.batchEmbedContents(model, requests);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<AbstractMessage> actualRequests = mockGenerativeService.getRequests();
    Assert.assertEquals(1, actualRequests.size());
    BatchEmbedContentsRequest actualRequest = ((BatchEmbedContentsRequest) actualRequests.get(0));

    Assert.assertEquals(model, actualRequest.getModel());
    Assert.assertEquals(requests, actualRequest.getRequestsList());
    Assert.assertTrue(
        channelProvider.isHeaderSent(
            ApiClientHeaderProvider.getDefaultApiClientHeaderKey(),
            GaxGrpcProperties.getDefaultApiClientHeaderPattern()));
  }

  @Test
  public void batchEmbedContentsExceptionTest2() throws Exception {
    StatusRuntimeException exception = new StatusRuntimeException(io.grpc.Status.INVALID_ARGUMENT);
    mockGenerativeService.addException(exception);

    try {
      String model = "model104069929";
      List<EmbedContentRequest> requests = new ArrayList<>();
      client.batchEmbedContents(model, requests);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void countTokensTest() throws Exception {
    CountTokensResponse expectedResponse =
        CountTokensResponse.newBuilder().setTotalTokens(730673909).build();
    mockGenerativeService.addResponse(expectedResponse);

    ModelName model = ModelName.of("[MODEL]");
    List<Content> contents = new ArrayList<>();

    CountTokensResponse actualResponse = client.countTokens(model, contents);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<AbstractMessage> actualRequests = mockGenerativeService.getRequests();
    Assert.assertEquals(1, actualRequests.size());
    CountTokensRequest actualRequest = ((CountTokensRequest) actualRequests.get(0));

    Assert.assertEquals(model.toString(), actualRequest.getModel());
    Assert.assertEquals(contents, actualRequest.getContentsList());
    Assert.assertTrue(
        channelProvider.isHeaderSent(
            ApiClientHeaderProvider.getDefaultApiClientHeaderKey(),
            GaxGrpcProperties.getDefaultApiClientHeaderPattern()));
  }

  @Test
  public void countTokensExceptionTest() throws Exception {
    StatusRuntimeException exception = new StatusRuntimeException(io.grpc.Status.INVALID_ARGUMENT);
    mockGenerativeService.addException(exception);

    try {
      ModelName model = ModelName.of("[MODEL]");
      List<Content> contents = new ArrayList<>();
      client.countTokens(model, contents);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }

  @Test
  public void countTokensTest2() throws Exception {
    CountTokensResponse expectedResponse =
        CountTokensResponse.newBuilder().setTotalTokens(730673909).build();
    mockGenerativeService.addResponse(expectedResponse);

    String model = "model104069929";
    List<Content> contents = new ArrayList<>();

    CountTokensResponse actualResponse = client.countTokens(model, contents);
    Assert.assertEquals(expectedResponse, actualResponse);

    List<AbstractMessage> actualRequests = mockGenerativeService.getRequests();
    Assert.assertEquals(1, actualRequests.size());
    CountTokensRequest actualRequest = ((CountTokensRequest) actualRequests.get(0));

    Assert.assertEquals(model, actualRequest.getModel());
    Assert.assertEquals(contents, actualRequest.getContentsList());
    Assert.assertTrue(
        channelProvider.isHeaderSent(
            ApiClientHeaderProvider.getDefaultApiClientHeaderKey(),
            GaxGrpcProperties.getDefaultApiClientHeaderPattern()));
  }

  @Test
  public void countTokensExceptionTest2() throws Exception {
    StatusRuntimeException exception = new StatusRuntimeException(io.grpc.Status.INVALID_ARGUMENT);
    mockGenerativeService.addException(exception);

    try {
      String model = "model104069929";
      List<Content> contents = new ArrayList<>();
      client.countTokens(model, contents);
      Assert.fail("No exception raised");
    } catch (InvalidArgumentException e) {
      // Expected exception.
    }
  }
}
