/*
 * Copyright 2023 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.ai.generativelanguage.v1;

import com.google.ai.generativelanguage.v1.GenerativeServiceGrpc.GenerativeServiceImplBase;
import com.google.api.core.BetaApi;
import com.google.protobuf.AbstractMessage;
import io.grpc.stub.StreamObserver;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import javax.annotation.Generated;

@BetaApi
@Generated("by gapic-generator-java")
public class MockGenerativeServiceImpl extends GenerativeServiceImplBase {
  private List<AbstractMessage> requests;
  private Queue<Object> responses;

  public MockGenerativeServiceImpl() {
    requests = new ArrayList<>();
    responses = new LinkedList<>();
  }

  public List<AbstractMessage> getRequests() {
    return requests;
  }

  public void addResponse(AbstractMessage response) {
    responses.add(response);
  }

  public void setResponses(List<AbstractMessage> responses) {
    this.responses = new LinkedList<Object>(responses);
  }

  public void addException(Exception exception) {
    responses.add(exception);
  }

  public void reset() {
    requests = new ArrayList<>();
    responses = new LinkedList<>();
  }

  @Override
  public void generateContent(
      GenerateContentRequest request, StreamObserver<GenerateContentResponse> responseObserver) {
    Object response = responses.poll();
    if (response instanceof GenerateContentResponse) {
      requests.add(request);
      responseObserver.onNext(((GenerateContentResponse) response));
      responseObserver.onCompleted();
    } else if (response instanceof Exception) {
      responseObserver.onError(((Exception) response));
    } else {
      responseObserver.onError(
          new IllegalArgumentException(
              String.format(
                  "Unrecognized response type %s for method GenerateContent, expected %s or %s",
                  response == null ? "null" : response.getClass().getName(),
                  GenerateContentResponse.class.getName(),
                  Exception.class.getName())));
    }
  }

  @Override
  public void streamGenerateContent(
      GenerateContentRequest request, StreamObserver<GenerateContentResponse> responseObserver) {
    Object response = responses.poll();
    if (response instanceof GenerateContentResponse) {
      requests.add(request);
      responseObserver.onNext(((GenerateContentResponse) response));
      responseObserver.onCompleted();
    } else if (response instanceof Exception) {
      responseObserver.onError(((Exception) response));
    } else {
      responseObserver.onError(
          new IllegalArgumentException(
              String.format(
                  "Unrecognized response type %s for method StreamGenerateContent, expected %s or %s",
                  response == null ? "null" : response.getClass().getName(),
                  GenerateContentResponse.class.getName(),
                  Exception.class.getName())));
    }
  }

  @Override
  public void embedContent(
      EmbedContentRequest request, StreamObserver<EmbedContentResponse> responseObserver) {
    Object response = responses.poll();
    if (response instanceof EmbedContentResponse) {
      requests.add(request);
      responseObserver.onNext(((EmbedContentResponse) response));
      responseObserver.onCompleted();
    } else if (response instanceof Exception) {
      responseObserver.onError(((Exception) response));
    } else {
      responseObserver.onError(
          new IllegalArgumentException(
              String.format(
                  "Unrecognized response type %s for method EmbedContent, expected %s or %s",
                  response == null ? "null" : response.getClass().getName(),
                  EmbedContentResponse.class.getName(),
                  Exception.class.getName())));
    }
  }

  @Override
  public void batchEmbedContents(
      BatchEmbedContentsRequest request,
      StreamObserver<BatchEmbedContentsResponse> responseObserver) {
    Object response = responses.poll();
    if (response instanceof BatchEmbedContentsResponse) {
      requests.add(request);
      responseObserver.onNext(((BatchEmbedContentsResponse) response));
      responseObserver.onCompleted();
    } else if (response instanceof Exception) {
      responseObserver.onError(((Exception) response));
    } else {
      responseObserver.onError(
          new IllegalArgumentException(
              String.format(
                  "Unrecognized response type %s for method BatchEmbedContents, expected %s or %s",
                  response == null ? "null" : response.getClass().getName(),
                  BatchEmbedContentsResponse.class.getName(),
                  Exception.class.getName())));
    }
  }

  @Override
  public void countTokens(
      CountTokensRequest request, StreamObserver<CountTokensResponse> responseObserver) {
    Object response = responses.poll();
    if (response instanceof CountTokensResponse) {
      requests.add(request);
      responseObserver.onNext(((CountTokensResponse) response));
      responseObserver.onCompleted();
    } else if (response instanceof Exception) {
      responseObserver.onError(((Exception) response));
    } else {
      responseObserver.onError(
          new IllegalArgumentException(
              String.format(
                  "Unrecognized response type %s for method CountTokens, expected %s or %s",
                  response == null ? "null" : response.getClass().getName(),
                  CountTokensResponse.class.getName(),
                  Exception.class.getName())));
    }
  }
}
